# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing import List
from typing_extensions import Literal, NotRequired, TypedDict


class PaymentMethodConfigurationUpdateParams(TypedDict):
    acss_debit: NotRequired["PaymentMethodConfigurationUpdateParamsAcssDebit"]
    """
    Canadian pre-authorized debit payments, check this [page](https://stripe.com/docs/payments/acss-debit) for more details like country availability.
    """
    active: NotRequired[bool]
    """
    Whether the configuration can be used for new payments.
    """
    affirm: NotRequired["PaymentMethodConfigurationUpdateParamsAffirm"]
    """
    [Affirm](https://www.affirm.com/) gives your customers a way to split purchases over a series of payments. Depending on the purchase, they can pay with four interest-free payments (Split Pay) or pay over a longer term (Installments), which might include interest. Check this [page](https://stripe.com/docs/payments/affirm) for more details like country availability.
    """
    afterpay_clearpay: NotRequired[
        "PaymentMethodConfigurationUpdateParamsAfterpayClearpay"
    ]
    """
    Afterpay gives your customers a way to pay for purchases in installments, check this [page](https://stripe.com/docs/payments/afterpay-clearpay) for more details like country availability. Afterpay is particularly popular among businesses selling fashion, beauty, and sports products.
    """
    alipay: NotRequired["PaymentMethodConfigurationUpdateParamsAlipay"]
    """
    Alipay is a digital wallet in China that has more than a billion active users worldwide. Alipay users can pay on the web or on a mobile device using login credentials or their Alipay app. Alipay has a low dispute rate and reduces fraud by authenticating payments using the customer's login credentials. Check this [page](https://stripe.com/docs/payments/alipay) for more details.
    """
    alma: NotRequired["PaymentMethodConfigurationUpdateParamsAlma"]
    """
    Alma is a Buy Now, Pay Later payment method that offers customers the ability to pay in 2, 3, or 4 installments.
    """
    amazon_pay: NotRequired["PaymentMethodConfigurationUpdateParamsAmazonPay"]
    """
    Amazon Pay is a wallet payment method that lets your customers check out the same way as on Amazon.
    """
    apple_pay: NotRequired["PaymentMethodConfigurationUpdateParamsApplePay"]
    """
    Stripe users can accept [Apple Pay](https://stripe.com/payments/apple-pay) in iOS applications in iOS 9 and later, and on the web in Safari starting with iOS 10 or macOS Sierra. There are no additional fees to process Apple Pay payments, and the [pricing](https://stripe.com/pricing) is the same as other card transactions. Check this [page](https://stripe.com/docs/apple-pay) for more details.
    """
    apple_pay_later: NotRequired[
        "PaymentMethodConfigurationUpdateParamsApplePayLater"
    ]
    """
    Apple Pay Later, a payment method for customers to buy now and pay later, gives your customers a way to split purchases into four installments across six weeks.
    """
    au_becs_debit: NotRequired[
        "PaymentMethodConfigurationUpdateParamsAuBecsDebit"
    ]
    """
    Stripe users in Australia can accept Bulk Electronic Clearing System (BECS) direct debit payments from customers with an Australian bank account. Check this [page](https://stripe.com/docs/payments/au-becs-debit) for more details.
    """
    bacs_debit: NotRequired["PaymentMethodConfigurationUpdateParamsBacsDebit"]
    """
    Stripe users in the UK can accept Bacs Direct Debit payments from customers with a UK bank account, check this [page](https://stripe.com/docs/payments/payment-methods/bacs-debit) for more details.
    """
    bancontact: NotRequired["PaymentMethodConfigurationUpdateParamsBancontact"]
    """
    Bancontact is the most popular online payment method in Belgium, with over 15 million cards in circulation. [Customers](https://stripe.com/docs/api/customers) use a Bancontact card or mobile app linked to a Belgian bank account to make online payments that are secure, guaranteed, and confirmed immediately. Check this [page](https://stripe.com/docs/payments/bancontact) for more details.
    """
    billie: NotRequired["PaymentMethodConfigurationUpdateParamsBillie"]
    """
    Billie is a [single-use](https://docs.stripe.com/payments/payment-methods#usage) payment method that offers businesses Pay by Invoice where they offer payment terms ranging from 7-120 days. Customers are redirected from your website or app, authorize the payment with Billie, then return to your website or app. You get [immediate notification](https://docs.stripe.com/payments/payment-methods#payment-notification) of whether the payment succeeded or failed.
    """
    blik: NotRequired["PaymentMethodConfigurationUpdateParamsBlik"]
    """
    BLIK is a [single use](https://stripe.com/docs/payments/payment-methods#usage) payment method that requires customers to authenticate their payments. When customers want to pay online using BLIK, they request a six-digit code from their banking application and enter it into the payment collection form. Check this [page](https://stripe.com/docs/payments/blik) for more details.
    """
    boleto: NotRequired["PaymentMethodConfigurationUpdateParamsBoleto"]
    """
    Boleto is an official (regulated by the Central Bank of Brazil) payment method in Brazil. Check this [page](https://stripe.com/docs/payments/boleto) for more details.
    """
    card: NotRequired["PaymentMethodConfigurationUpdateParamsCard"]
    """
    Cards are a popular way for consumers and businesses to pay online or in person. Stripe supports global and local card networks.
    """
    cartes_bancaires: NotRequired[
        "PaymentMethodConfigurationUpdateParamsCartesBancaires"
    ]
    """
    Cartes Bancaires is France's local card network. More than 95% of these cards are co-branded with either Visa or Mastercard, meaning you can process these cards over either Cartes Bancaires or the Visa or Mastercard networks. Check this [page](https://stripe.com/docs/payments/cartes-bancaires) for more details.
    """
    cashapp: NotRequired["PaymentMethodConfigurationUpdateParamsCashapp"]
    """
    Cash App is a popular consumer app in the US that allows customers to bank, invest, send, and receive money using their digital wallet. Check this [page](https://stripe.com/docs/payments/cash-app-pay) for more details.
    """
    crypto: NotRequired["PaymentMethodConfigurationUpdateParamsCrypto"]
    """
    [Stablecoin payments](https://stripe.com/docs/payments/stablecoin-payments) enable customers to pay in stablecoins like USDC from 100s of wallets including Phantom and Metamask.
    """
    customer_balance: NotRequired[
        "PaymentMethodConfigurationUpdateParamsCustomerBalance"
    ]
    """
    Uses a customer's [cash balance](https://stripe.com/docs/payments/customer-balance) for the payment. The cash balance can be funded via a bank transfer. Check this [page](https://stripe.com/docs/payments/bank-transfers) for more details.
    """
    eps: NotRequired["PaymentMethodConfigurationUpdateParamsEps"]
    """
    EPS is an Austria-based payment method that allows customers to complete transactions online using their bank credentials. EPS is supported by all Austrian banks and is accepted by over 80% of Austrian online retailers. Check this [page](https://stripe.com/docs/payments/eps) for more details.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    fpx: NotRequired["PaymentMethodConfigurationUpdateParamsFpx"]
    """
    Financial Process Exchange (FPX) is a Malaysia-based payment method that allows customers to complete transactions online using their bank credentials. Bank Negara Malaysia (BNM), the Central Bank of Malaysia, and eleven other major Malaysian financial institutions are members of the PayNet Group, which owns and operates FPX. It is one of the most popular online payment methods in Malaysia, with nearly 90 million transactions in 2018 according to BNM. Check this [page](https://stripe.com/docs/payments/fpx) for more details.
    """
    fr_meal_voucher_conecs: NotRequired[
        "PaymentMethodConfigurationUpdateParamsFrMealVoucherConecs"
    ]
    """
    Meal vouchers in France, or “titres-restaurant”, is a local benefits program commonly offered by employers for their employees to purchase prepared food and beverages on working days. Check this [page](https://stripe.com/docs/payments/benefits/fr-meal-vouchers) for more details.
    """
    giropay: NotRequired["PaymentMethodConfigurationUpdateParamsGiropay"]
    """
    giropay is a German payment method based on online banking, introduced in 2006. It allows customers to complete transactions online using their online banking environment, with funds debited from their bank account. Depending on their bank, customers confirm payments on giropay using a second factor of authentication or a PIN. giropay accounts for 10% of online checkouts in Germany. Check this [page](https://stripe.com/docs/payments/giropay) for more details.
    """
    google_pay: NotRequired["PaymentMethodConfigurationUpdateParamsGooglePay"]
    """
    Google Pay allows customers to make payments in your app or website using any credit or debit card saved to their Google Account, including those from Google Play, YouTube, Chrome, or an Android device. Use the Google Pay API to request any credit or debit card stored in your customer's Google account. Check this [page](https://stripe.com/docs/google-pay) for more details.
    """
    grabpay: NotRequired["PaymentMethodConfigurationUpdateParamsGrabpay"]
    """
    GrabPay is a payment method developed by [Grab](https://www.grab.com/sg/consumer/finance/pay/). GrabPay is a digital wallet - customers maintain a balance in their wallets that they pay out with. Check this [page](https://stripe.com/docs/payments/grabpay) for more details.
    """
    ideal: NotRequired["PaymentMethodConfigurationUpdateParamsIdeal"]
    """
    iDEAL is a Netherlands-based payment method that allows customers to complete transactions online using their bank credentials. All major Dutch banks are members of Currence, the scheme that operates iDEAL, making it the most popular online payment method in the Netherlands with a share of online transactions close to 55%. Check this [page](https://stripe.com/docs/payments/ideal) for more details.
    """
    jcb: NotRequired["PaymentMethodConfigurationUpdateParamsJcb"]
    """
    JCB is a credit card company based in Japan. JCB is currently available in Japan to businesses approved by JCB, and available to all businesses in Australia, Canada, Hong Kong, Japan, New Zealand, Singapore, Switzerland, United Kingdom, United States, and all countries in the European Economic Area except Iceland. Check this [page](https://support.stripe.com/questions/accepting-japan-credit-bureau-%28jcb%29-payments) for more details.
    """
    kakao_pay: NotRequired["PaymentMethodConfigurationUpdateParamsKakaoPay"]
    """
    Kakao Pay is a popular local wallet available in South Korea.
    """
    klarna: NotRequired["PaymentMethodConfigurationUpdateParamsKlarna"]
    """
    Klarna gives customers a range of [payment options](https://stripe.com/docs/payments/klarna#payment-options) during checkout. Available payment options vary depending on the customer's billing address and the transaction amount. These payment options make it convenient for customers to purchase items in all price ranges. Check this [page](https://stripe.com/docs/payments/klarna) for more details.
    """
    konbini: NotRequired["PaymentMethodConfigurationUpdateParamsKonbini"]
    """
    Konbini allows customers in Japan to pay for bills and online purchases at convenience stores with cash. Check this [page](https://stripe.com/docs/payments/konbini) for more details.
    """
    kr_card: NotRequired["PaymentMethodConfigurationUpdateParamsKrCard"]
    """
    Korean cards let users pay using locally issued cards from South Korea.
    """
    link: NotRequired["PaymentMethodConfigurationUpdateParamsLink"]
    """
    [Link](https://stripe.com/docs/payments/link) is a payment method network. With Link, users save their payment details once, then reuse that information to pay with one click for any business on the network.
    """
    mb_way: NotRequired["PaymentMethodConfigurationUpdateParamsMbWay"]
    """
    MB WAY is the most popular wallet in Portugal. After entering their phone number in your checkout, customers approve the payment directly in their MB WAY app. Check this [page](https://stripe.com/docs/payments/mb-way) for more details.
    """
    mobilepay: NotRequired["PaymentMethodConfigurationUpdateParamsMobilepay"]
    """
    MobilePay is a [single-use](https://stripe.com/docs/payments/payment-methods#usage) card wallet payment method used in Denmark and Finland. It allows customers to [authenticate and approve](https://stripe.com/docs/payments/payment-methods#customer-actions) payments using the MobilePay app. Check this [page](https://stripe.com/docs/payments/mobilepay) for more details.
    """
    multibanco: NotRequired["PaymentMethodConfigurationUpdateParamsMultibanco"]
    """
    Stripe users in Europe and the United States can accept Multibanco payments from customers in Portugal using [Sources](https://stripe.com/docs/sources)—a single integration path for creating payments using any supported method.
    """
    name: NotRequired[str]
    """
    Configuration name.
    """
    naver_pay: NotRequired["PaymentMethodConfigurationUpdateParamsNaverPay"]
    """
    Naver Pay is a popular local wallet available in South Korea.
    """
    nz_bank_account: NotRequired[
        "PaymentMethodConfigurationUpdateParamsNzBankAccount"
    ]
    """
    Stripe users in New Zealand can accept Bulk Electronic Clearing System (BECS) direct debit payments from customers with a New Zeland bank account. Check this [page](https://stripe.com/docs/payments/nz-bank-account) for more details.
    """
    oxxo: NotRequired["PaymentMethodConfigurationUpdateParamsOxxo"]
    """
    OXXO is a Mexican chain of convenience stores with thousands of locations across Latin America and represents nearly 20% of online transactions in Mexico. OXXO allows customers to pay bills and online purchases in-store with cash. Check this [page](https://stripe.com/docs/payments/oxxo) for more details.
    """
    p24: NotRequired["PaymentMethodConfigurationUpdateParamsP24"]
    """
    Przelewy24 is a Poland-based payment method aggregator that allows customers to complete transactions online using bank transfers and other methods. Bank transfers account for 30% of online payments in Poland and Przelewy24 provides a way for customers to pay with over 165 banks. Check this [page](https://stripe.com/docs/payments/p24) for more details.
    """
    pay_by_bank: NotRequired["PaymentMethodConfigurationUpdateParamsPayByBank"]
    """
    Pay by bank is a redirect payment method backed by bank transfers. A customer is redirected to their bank to authorize a bank transfer for a given amount. This removes a lot of the error risks inherent in waiting for the customer to initiate a transfer themselves, and is less expensive than card payments.
    """
    payco: NotRequired["PaymentMethodConfigurationUpdateParamsPayco"]
    """
    PAYCO is a [single-use](https://docs.stripe.com/payments/payment-methods#usage local wallet available in South Korea.
    """
    paynow: NotRequired["PaymentMethodConfigurationUpdateParamsPaynow"]
    """
    PayNow is a Singapore-based payment method that allows customers to make a payment using their preferred app from participating banks and participating non-bank financial institutions. Check this [page](https://stripe.com/docs/payments/paynow) for more details.
    """
    paypal: NotRequired["PaymentMethodConfigurationUpdateParamsPaypal"]
    """
    PayPal, a digital wallet popular with customers in Europe, allows your customers worldwide to pay using their PayPal account. Check this [page](https://stripe.com/docs/payments/paypal) for more details.
    """
    pix: NotRequired["PaymentMethodConfigurationUpdateParamsPix"]
    """
    Pix is a payment method popular in Brazil. When paying with Pix, customers authenticate and approve payments by scanning a QR code in their preferred banking app. Check this [page](https://docs.stripe.com/payments/pix) for more details.
    """
    promptpay: NotRequired["PaymentMethodConfigurationUpdateParamsPromptpay"]
    """
    PromptPay is a Thailand-based payment method that allows customers to make a payment using their preferred app from participating banks. Check this [page](https://stripe.com/docs/payments/promptpay) for more details.
    """
    revolut_pay: NotRequired[
        "PaymentMethodConfigurationUpdateParamsRevolutPay"
    ]
    """
    Revolut Pay, developed by Revolut, a global finance app, is a digital wallet payment method. Revolut Pay uses the customer's stored balance or cards to fund the payment, and offers the option for non-Revolut customers to save their details after their first purchase.
    """
    samsung_pay: NotRequired[
        "PaymentMethodConfigurationUpdateParamsSamsungPay"
    ]
    """
    Samsung Pay is a [single-use](https://docs.stripe.com/payments/payment-methods#usage local wallet available in South Korea.
    """
    satispay: NotRequired["PaymentMethodConfigurationUpdateParamsSatispay"]
    """
    Satispay is a [single-use](https://docs.stripe.com/payments/payment-methods#usage) payment method where customers are required to [authenticate](https://docs.stripe.com/payments/payment-methods#customer-actions) their payment. Customers pay by being redirected from your website or app, authorizing the payment with Satispay, then returning to your website or app. You get [immediate notification](https://docs.stripe.com/payments/payment-methods#payment-notification) of whether the payment succeeded or failed.
    """
    sepa_debit: NotRequired["PaymentMethodConfigurationUpdateParamsSepaDebit"]
    """
    The [Single Euro Payments Area (SEPA)](https://en.wikipedia.org/wiki/Single_Euro_Payments_Area) is an initiative of the European Union to simplify payments within and across member countries. SEPA established and enforced banking standards to allow for the direct debiting of every EUR-denominated bank account within the SEPA region, check this [page](https://stripe.com/docs/payments/sepa-debit) for more details.
    """
    sofort: NotRequired["PaymentMethodConfigurationUpdateParamsSofort"]
    """
    Stripe users in Europe and the United States can use the [Payment Intents API](https://stripe.com/docs/payments/payment-intents)—a single integration path for creating payments using any supported method—to accept [Sofort](https://www.sofort.com/) payments from customers. Check this [page](https://stripe.com/docs/payments/sofort) for more details.
    """
    swish: NotRequired["PaymentMethodConfigurationUpdateParamsSwish"]
    """
    Swish is a [real-time](https://stripe.com/docs/payments/real-time) payment method popular in Sweden. It allows customers to [authenticate and approve](https://stripe.com/docs/payments/payment-methods#customer-actions) payments using the Swish mobile app and the Swedish BankID mobile app. Check this [page](https://stripe.com/docs/payments/swish) for more details.
    """
    twint: NotRequired["PaymentMethodConfigurationUpdateParamsTwint"]
    """
    Twint is a payment method popular in Switzerland. It allows customers to pay using their mobile phone. Check this [page](https://docs.stripe.com/payments/twint) for more details.
    """
    us_bank_account: NotRequired[
        "PaymentMethodConfigurationUpdateParamsUsBankAccount"
    ]
    """
    Stripe users in the United States can accept ACH direct debit payments from customers with a US bank account using the Automated Clearing House (ACH) payments system operated by Nacha. Check this [page](https://stripe.com/docs/payments/ach-direct-debit) for more details.
    """
    wechat_pay: NotRequired["PaymentMethodConfigurationUpdateParamsWechatPay"]
    """
    WeChat, owned by Tencent, is China's leading mobile app with over 1 billion monthly active users. Chinese consumers can use WeChat Pay to pay for goods and services inside of businesses' apps and websites. WeChat Pay users buy most frequently in gaming, e-commerce, travel, online education, and food/nutrition. Check this [page](https://stripe.com/docs/payments/wechat-pay) for more details.
    """
    zip: NotRequired["PaymentMethodConfigurationUpdateParamsZip"]
    """
    Zip gives your customers a way to split purchases over a series of payments. Check this [page](https://stripe.com/docs/payments/zip) for more details like country availability.
    """


class PaymentMethodConfigurationUpdateParamsAcssDebit(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsAcssDebitDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsAcssDebitDisplayPreference(
    TypedDict,
):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsAffirm(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsAffirmDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsAffirmDisplayPreference(TypedDict):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsAfterpayClearpay(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsAfterpayClearpayDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsAfterpayClearpayDisplayPreference(
    TypedDict,
):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsAlipay(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsAlipayDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsAlipayDisplayPreference(TypedDict):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsAlma(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsAlmaDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsAlmaDisplayPreference(TypedDict):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsAmazonPay(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsAmazonPayDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsAmazonPayDisplayPreference(
    TypedDict,
):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsApplePay(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsApplePayDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsApplePayDisplayPreference(
    TypedDict,
):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsApplePayLater(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsApplePayLaterDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsApplePayLaterDisplayPreference(
    TypedDict,
):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsAuBecsDebit(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsAuBecsDebitDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsAuBecsDebitDisplayPreference(
    TypedDict,
):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsBacsDebit(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsBacsDebitDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsBacsDebitDisplayPreference(
    TypedDict,
):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsBancontact(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsBancontactDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsBancontactDisplayPreference(
    TypedDict,
):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsBillie(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsBillieDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsBillieDisplayPreference(TypedDict):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsBlik(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsBlikDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsBlikDisplayPreference(TypedDict):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsBoleto(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsBoletoDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsBoletoDisplayPreference(TypedDict):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsCard(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsCardDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsCardDisplayPreference(TypedDict):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsCartesBancaires(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsCartesBancairesDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsCartesBancairesDisplayPreference(
    TypedDict,
):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsCashapp(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsCashappDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsCashappDisplayPreference(
    TypedDict
):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsCrypto(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsCryptoDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsCryptoDisplayPreference(TypedDict):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsCustomerBalance(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsCustomerBalanceDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsCustomerBalanceDisplayPreference(
    TypedDict,
):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsEps(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsEpsDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsEpsDisplayPreference(TypedDict):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsFpx(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsFpxDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsFpxDisplayPreference(TypedDict):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsFrMealVoucherConecs(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsFrMealVoucherConecsDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsFrMealVoucherConecsDisplayPreference(
    TypedDict,
):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsGiropay(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsGiropayDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsGiropayDisplayPreference(
    TypedDict
):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsGooglePay(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsGooglePayDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsGooglePayDisplayPreference(
    TypedDict,
):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsGrabpay(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsGrabpayDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsGrabpayDisplayPreference(
    TypedDict
):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsIdeal(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsIdealDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsIdealDisplayPreference(TypedDict):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsJcb(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsJcbDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsJcbDisplayPreference(TypedDict):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsKakaoPay(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsKakaoPayDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsKakaoPayDisplayPreference(
    TypedDict,
):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsKlarna(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsKlarnaDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsKlarnaDisplayPreference(TypedDict):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsKonbini(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsKonbiniDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsKonbiniDisplayPreference(
    TypedDict
):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsKrCard(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsKrCardDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsKrCardDisplayPreference(TypedDict):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsLink(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsLinkDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsLinkDisplayPreference(TypedDict):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsMbWay(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsMbWayDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsMbWayDisplayPreference(TypedDict):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsMobilepay(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsMobilepayDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsMobilepayDisplayPreference(
    TypedDict,
):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsMultibanco(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsMultibancoDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsMultibancoDisplayPreference(
    TypedDict,
):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsNaverPay(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsNaverPayDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsNaverPayDisplayPreference(
    TypedDict,
):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsNzBankAccount(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsNzBankAccountDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsNzBankAccountDisplayPreference(
    TypedDict,
):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsOxxo(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsOxxoDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsOxxoDisplayPreference(TypedDict):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsP24(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsP24DisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsP24DisplayPreference(TypedDict):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsPayByBank(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsPayByBankDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsPayByBankDisplayPreference(
    TypedDict,
):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsPayco(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsPaycoDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsPaycoDisplayPreference(TypedDict):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsPaynow(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsPaynowDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsPaynowDisplayPreference(TypedDict):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsPaypal(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsPaypalDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsPaypalDisplayPreference(TypedDict):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsPix(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsPixDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsPixDisplayPreference(TypedDict):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsPromptpay(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsPromptpayDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsPromptpayDisplayPreference(
    TypedDict,
):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsRevolutPay(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsRevolutPayDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsRevolutPayDisplayPreference(
    TypedDict,
):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsSamsungPay(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsSamsungPayDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsSamsungPayDisplayPreference(
    TypedDict,
):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsSatispay(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsSatispayDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsSatispayDisplayPreference(
    TypedDict,
):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsSepaDebit(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsSepaDebitDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsSepaDebitDisplayPreference(
    TypedDict,
):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsSofort(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsSofortDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsSofortDisplayPreference(TypedDict):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsSwish(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsSwishDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsSwishDisplayPreference(TypedDict):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsTwint(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsTwintDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsTwintDisplayPreference(TypedDict):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsUsBankAccount(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsUsBankAccountDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsUsBankAccountDisplayPreference(
    TypedDict,
):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsWechatPay(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsWechatPayDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsWechatPayDisplayPreference(
    TypedDict,
):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """


class PaymentMethodConfigurationUpdateParamsZip(TypedDict):
    display_preference: NotRequired[
        "PaymentMethodConfigurationUpdateParamsZipDisplayPreference"
    ]
    """
    Whether or not the payment method should be displayed.
    """


class PaymentMethodConfigurationUpdateParamsZipDisplayPreference(TypedDict):
    preference: NotRequired[Literal["none", "off", "on"]]
    """
    The account's preference for whether or not to display this payment method.
    """



# pyright: strict
import datetime
import json
from copy import deepcopy
from typing_extensions import TYPE_CHECKING, Type, Literal, Self
from typing import (
    Any,
    Dict,
    List,
    Optional,
    Mapping,
    Set,
    Tuple,
    ClassVar,
    Union,
    cast,
    overload,
)

# Used to break circular imports
import stripe  # noqa: IMP101
from stripe import _util

from stripe._stripe_response import (
    StripeResponse,
    StripeStreamResponse,
    StripeStreamResponseAsync,
)
from stripe._encode import _encode_datetime  # pyright: ignore
from stripe._request_options import (
    PERSISTENT_OPTIONS_KEYS,
    extract_options_from_dict,
)
from stripe._api_mode import ApiMode
from stripe._base_address import BaseAddress

if TYPE_CHECKING:
    from stripe._api_requestor import _APIRequestor  # pyright: ignore[reportPrivateUsage]


@overload
def _compute_diff(
    current: Dict[str, Any], previous: Optional[Dict[str, Any]]
) -> Dict[str, Any]: ...


@overload
def _compute_diff(
    current: object, previous: Optional[Dict[str, Any]]
) -> object: ...


def _compute_diff(
    current: object, previous: Optional[Dict[str, Any]]
) -> object:
    if isinstance(current, dict):
        current = cast(Dict[str, Any], current)
        previous = previous or {}
        diff = current.copy()
        for key in set(previous.keys()) - set(diff.keys()):
            diff[key] = ""
        return diff
    return current if current is not None else ""


def _serialize_list(
    array: Optional[List[Any]], previous: List[Any]
) -> Dict[str, Any]:
    array = array or []
    previous = previous or []
    params: Dict[str, Any] = {}

    for i, v in enumerate(array):
        previous_item = previous[i] if len(previous) > i else None
        if hasattr(v, "serialize"):
            params[str(i)] = v.serialize(previous_item)
        else:
            params[str(i)] = _compute_diff(v, previous_item)

    return params


class StripeObject(Dict[str, Any]):
    class _ReprJSONEncoder(json.JSONEncoder):
        def default(self, o: Any) -> Any:
            if isinstance(o, datetime.datetime):
                return _encode_datetime(o)
            return super(StripeObject._ReprJSONEncoder, self).default(o)

    _retrieve_params: Mapping[str, Any]
    _previous: Optional[Mapping[str, Any]]

    def __init__(
        self,
        id: Optional[str] = None,
        api_key: Optional[str] = None,
        stripe_version: Optional[str] = None,
        stripe_account: Optional[str] = None,
        last_response: Optional[StripeResponse] = None,
        *,
        _requestor: Optional["_APIRequestor"] = None,
        # TODO: is a more specific type possible here?
        **params: Any,
    ):
        super(StripeObject, self).__init__()

        self._unsaved_values: Set[str] = set()
        self._transient_values: Set[str] = set()
        self._last_response = last_response

        self._retrieve_params = params
        self._previous = None

        from stripe._api_requestor import _APIRequestor  # pyright: ignore[reportPrivateUsage]

        self._requestor = (
            _APIRequestor._global_with_options(  # pyright: ignore[reportPrivateUsage]
                api_key=api_key,
                stripe_version=stripe_version,
                stripe_account=stripe_account,
            )
            if _requestor is None
            else _requestor
        )

        if id:
            self["id"] = id

    @property
    def api_key(self):
        return self._requestor.api_key

    @property
    def stripe_account(self):
        return self._requestor.stripe_account

    @property
    def stripe_version(self):
        return self._requestor.stripe_version

    @property
    def last_response(self) -> Optional[StripeResponse]:
        return self._last_response

    # StripeObject inherits from `dict` which has an update method, and this doesn't quite match
    # the full signature of the update method in MutableMapping. But we ignore.
    def update(  # pyright: ignore
        self, update_dict: Mapping[str, Any]
    ) -> None:
        for k in update_dict:
            self._unsaved_values.add(k)

        return super(StripeObject, self).update(update_dict)

    if not TYPE_CHECKING:

        def __setattr__(self, k, v):
            if k in PERSISTENT_OPTIONS_KEYS:
                self._requestor = self._requestor._new_requestor_with_options(
                    {k: v}
                )
                return None

            if k[0] == "_" or k in self.__dict__:
                return super(StripeObject, self).__setattr__(k, v)

            self[k] = v
            return None

        def __getattr__(self, k):
            if k[0] == "_":
                raise AttributeError(k)

            try:
                if k in self._field_remappings:
                    k = self._field_remappings[k]
                return self[k]
            except KeyError as err:
                raise AttributeError(*err.args) from err

        def __delattr__(self, k):
            if k[0] == "_" or k in self.__dict__:
                return super(StripeObject, self).__delattr__(k)
            else:
                del self[k]

    def __setitem__(self, k: str, v: Any) -> None:
        if v == "":
            raise ValueError(
                "You cannot set %s to an empty string on this object. "
                "The empty string is treated specially in our requests. "
                "If you'd like to delete the property using the save() method on this object, you may set %s.%s=None. "
                "Alternatively, you can pass %s='' to delete the property when using a resource method such as modify()."
                % (k, str(self), k, k)
            )

        # Allows for unpickling in Python 3.x
        if not hasattr(self, "_unsaved_values"):
            self._unsaved_values = set()

        self._unsaved_values.add(k)

        super(StripeObject, self).__setitem__(k, v)

    def __getitem__(self, k: str) -> Any:
        try:
            return super(StripeObject, self).__getitem__(k)
        except KeyError as err:
            if k in self._transient_values:
                raise KeyError(
                    "%r.  HINT: The %r attribute was set in the past."
                    "It was then wiped when refreshing the object with "
                    "the result returned by Stripe's API, probably as a "
                    "result of a save().  The attributes currently "
                    "available on this object are: %s"
                    % (k, k, ", ".join(list(self.keys())))
                )
            else:
                raise err

    def __delitem__(self, k: str) -> None:
        super(StripeObject, self).__delitem__(k)

        # Allows for unpickling in Python 3.x
        if hasattr(self, "_unsaved_values") and k in self._unsaved_values:
            self._unsaved_values.remove(k)

    # Custom unpickling method that uses `update` to update the dictionary
    # without calling __setitem__, which would fail if any value is an empty
    # string
    def __setstate__(self, state: Dict[str, Any]) -> None:
        self.update(state)

    # Custom pickling method to ensure the instance is pickled as a custom
    # class and not as a dict, otherwise __setstate__ would not be called when
    # unpickling.
    def __reduce__(self) -> Tuple[Any, ...]:
        reduce_value = (
            type(self),  # callable
            (  # args
                self.get("id", None),
                self.api_key,
                self.stripe_version,
                self.stripe_account,
            ),
            dict(self),  # state
        )
        return reduce_value

    @classmethod
    def construct_from(
        cls,
        values: Dict[str, Any],
        key: Optional[str],
        stripe_version: Optional[str] = None,
        stripe_account: Optional[str] = None,
        last_response: Optional[StripeResponse] = None,
        *,
        api_mode: ApiMode = "V1",
    ) -> Self:
        from stripe._api_requestor import _APIRequestor  # pyright: ignore[reportPrivateUsage]

        return cls._construct_from(
            values=values,
            requestor=_APIRequestor._global_with_options(  # pyright: ignore[reportPrivateUsage]
                api_key=key,
                stripe_version=stripe_version,
                stripe_account=stripe_account,
            ),
            api_mode=api_mode,
            last_response=last_response,
        )

    @classmethod
    def _construct_from(
        cls,
        *,
        values: Dict[str, Any],
        last_response: Optional[StripeResponse] = None,
        requestor: "_APIRequestor",
        api_mode: ApiMode,
    ) -> Self:
        instance = cls(
            values.get("id"),
            last_response=last_response,
            _requestor=requestor,
        )
        instance._refresh_from(
            values=values,
            last_response=last_response,
            requestor=requestor,
            api_mode=api_mode,
        )
        return instance

    def refresh_from(
        self,
        values: Dict[str, Any],
        api_key: Optional[str] = None,
        partial: Optional[bool] = False,
        stripe_version: Optional[str] = None,
        stripe_account: Optional[str] = None,
        last_response: Optional[StripeResponse] = None,
        *,
        api_mode: ApiMode = "V1",
    ) -> None:
        self._refresh_from(
            values=values,
            partial=partial,
            last_response=last_response,
            requestor=self._requestor._new_requestor_with_options(  # pyright: ignore[reportPrivateUsage]
                {
                    "api_key": api_key,
                    "stripe_version": stripe_version,
                    "stripe_account": stripe_account,
                }
            ),
            api_mode=api_mode,
        )

    def _refresh_from(
        self,
        *,
        values: Dict[str, Any],
        partial: Optional[bool] = False,
        last_response: Optional[StripeResponse] = None,
        requestor: Optional["_APIRequestor"] = None,
        api_mode: ApiMode,
    ) -> None:
        self._requestor = requestor or self._requestor
        self._last_response = last_response or getattr(
            values, "_last_response", None
        )

        # Wipe old state before setting new.  This is useful for e.g.
        # updating a customer, where there is no persistent card
        # parameter.  Mark those values which don't persist as transient
        if partial:
            self._unsaved_values = self._unsaved_values - set(values)
        else:
            removed = set(self.keys()) - set(values)
            self._transient_values = self._transient_values | removed
            self._unsaved_values = set()
            self.clear()

        self._transient_values = self._transient_values - set(values)

        for k, v in values.items():
            inner_class = self._get_inner_class_type(k)
            is_dict = self._get_inner_class_is_beneath_dict(k)
            if is_dict:
                obj = {
                    k: None
                    if v is None
                    else cast(
                        StripeObject,
                        _util._convert_to_stripe_object(  # pyright: ignore[reportPrivateUsage]
                            resp=v,
                            params=None,
                            klass_=inner_class,
                            requestor=self._requestor,
                            api_mode=api_mode,
                        ),
                    )
                    for k, v in v.items()
                }
            else:
                obj = cast(
                    Union[StripeObject, List[StripeObject]],
                    _util._convert_to_stripe_object(  # pyright: ignore[reportPrivateUsage]
                        resp=v,
                        params=None,
                        klass_=inner_class,
                        requestor=self._requestor,
                        api_mode=api_mode,
                    ),
                )
            super(StripeObject, self).__setitem__(k, obj)

        self._previous = values

    @_util.deprecated(
        "This will be removed in a future version of stripe-python."
    )
    def request(
        self,
        method: Literal["get", "post", "delete"],
        url: str,
        params: Optional[Dict[str, Any]] = None,
        *,
        base_address: BaseAddress = "api",
    ) -> "StripeObject":
        return StripeObject._request(
            self,
            method,
            url,
            params=params,
            base_address=base_address,
        )

    def _request(
        self,
        method: Literal["get", "post", "delete"],
        url: str,
        params: Optional[Mapping[str, Any]] = None,
        usage: Optional[List[str]] = None,
        *,
        base_address: BaseAddress,
    ) -> "StripeObject":
        if params is None:
            params = self._retrieve_params

        request_options, request_params = extract_options_from_dict(params)

        return self._requestor.request(
            method,
            url,
            params=request_params,
            options=request_options,
            base_address=base_address,
            usage=usage,
        )

    async def _request_async(
        self,
        method: Literal["get", "post", "delete"],
        url: str,
        params: Optional[Mapping[str, Any]] = None,
        usage: Optional[List[str]] = None,
        *,
        base_address: BaseAddress,
    ) -> "StripeObject":
        if params is None:
            params = self._retrieve_params

        request_options, request_params = extract_options_from_dict(params)

        return await self._requestor.request_async(
            method,
            url,
            params=request_params,
            options=request_options,
            base_address=base_address,
            usage=usage,
        )

    def _request_stream(
        self,
        method: str,
        url: str,
        params: Optional[Mapping[str, Any]] = None,
        *,
        base_address: BaseAddress = "api",
    ) -> StripeStreamResponse:
        if params is None:
            params = self._retrieve_params

        request_options, request_params = extract_options_from_dict(params)
        return self._requestor.request_stream(
            method,
            url,
            params=request_params,
            options=request_options,
            base_address=base_address,
        )

    async def _request_stream_async(
        self,
        method: str,
        url: str,
        params: Optional[Mapping[str, Any]] = None,
        *,
        base_address: BaseAddress = "api",
    ) -> StripeStreamResponseAsync:
        if params is None:
            params = self._retrieve_params

        request_options, request_params = extract_options_from_dict(params)
        return await self._requestor.request_stream_async(
            method,
            url,
            params=request_params,
            options=request_options,
            base_address=base_address,
        )

    def __repr__(self) -> str:
        ident_parts = [type(self).__name__]

        obj_str = self.get("object")
        if isinstance(obj_str, str):
            ident_parts.append(obj_str)

        if isinstance(self.get("id"), str):
            ident_parts.append("id=%s" % (self.get("id"),))

        unicode_repr = "<%s at %s> JSON: %s" % (
            " ".join(ident_parts),
            hex(id(self)),
            str(self),
        )
        return unicode_repr

    def __str__(self) -> str:
        return json.dumps(
            self._to_dict_recursive(),
            sort_keys=True,
            indent=2,
            cls=self._ReprJSONEncoder,
        )

    @_util.deprecated(
        "Deprecated. The public interface will be removed in a future version."
    )
    def to_dict(self) -> Dict[str, Any]:
        return dict(self)

    def _to_dict_recursive(self) -> Dict[str, Any]:
        def maybe_to_dict_recursive(
            value: Optional[Union[StripeObject, Dict[str, Any]]],
        ) -> Optional[Dict[str, Any]]:
            if value is None:
                return None
            elif isinstance(value, StripeObject):
                return value._to_dict_recursive()
            else:
                return value

        return {
            key: list(map(maybe_to_dict_recursive, cast(List[Any], value)))
            if isinstance(value, list)
            else maybe_to_dict_recursive(value)
            for key, value in dict(self).items()
        }

    @_util.deprecated(
        "For internal stripe-python use only. The public interface will be removed in a future version."
    )
    def to_dict_recursive(self) -> Dict[str, Any]:
        return self._to_dict_recursive()

    @property
    @_util.deprecated(
        "For internal stripe-python use only. The public interface will be removed in a future version."
    )
    def stripe_id(self) -> Optional[str]:
        return getattr(self, "id")

    def serialize(
        self, previous: Optional[Mapping[str, Any]]
    ) -> Dict[str, Any]:
        params: Dict[str, Any] = {}
        unsaved_keys = self._unsaved_values or set()
        previous = previous or self._previous or {}

        for k, v in self.items():
            if k == "id" or k.startswith("_"):
                continue
            elif isinstance(v, stripe.APIResource):
                continue
            elif hasattr(v, "serialize"):
                child = v.serialize(previous.get(k, None))
                if child != {}:
                    params[k] = child
            elif k in unsaved_keys:
                params[k] = _compute_diff(v, previous.get(k, None))
            elif k == "additional_owners" and v is not None:
                params[k] = _serialize_list(v, previous.get(k, None))

        return params

    # This class overrides __setitem__ to throw exceptions on inputs that it
    # doesn't like. This can cause problems when we try to copy an object
    # wholesale because some data that's returned from the API may not be valid
    # if it was set to be set manually. Here we override the class' copy
    # arguments so that we can bypass these possible exceptions on __setitem__.
    def __copy__(self) -> "StripeObject":
        copied = StripeObject(
            self.get("id"),
            self.api_key,
            stripe_version=self.stripe_version,
            stripe_account=self.stripe_account,
        )

        copied._retrieve_params = self._retrieve_params

        for k, v in self.items():
            # Call parent's __setitem__ to avoid checks that we've added in the
            # overridden version that can throw exceptions.
            super(StripeObject, copied).__setitem__(k, v)

        return copied

    # This class overrides __setitem__ to throw exceptions on inputs that it
    # doesn't like. This can cause problems when we try to copy an object
    # wholesale because some data that's returned from the API may not be valid
    # if it was set to be set manually. Here we override the class' copy
    # arguments so that we can bypass these possible exceptions on __setitem__.
    def __deepcopy__(self, memo: Dict[int, Any]) -> "StripeObject":
        copied = self.__copy__()
        memo[id(self)] = copied

        for k, v in self.items():
            # Call parent's __setitem__ to avoid checks that we've added in the
            # overridden version that can throw exceptions.
            super(StripeObject, copied).__setitem__(k, deepcopy(v, memo))

        return copied

    _field_remappings: ClassVar[Dict[str, str]] = {}

    _inner_class_types: ClassVar[Dict[str, Type["StripeObject"]]] = {}
    _inner_class_dicts: ClassVar[List[str]] = []

    def _get_inner_class_type(
        self, field_name: str
    ) -> Optional[Type["StripeObject"]]:
        return self._inner_class_types.get(field_name)

    def _get_inner_class_is_beneath_dict(self, field_name: str):
        return field_name in self._inner_class_dicts



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._createable_api_resource import CreateableAPIResource
from stripe._expandable_field import ExpandableField
from stripe._list_object import ListObject
from stripe._listable_api_resource import ListableAPIResource
from stripe._stripe_object import StripeObject
from stripe._updateable_api_resource import UpdateableAPIResource
from stripe._util import class_method_variant, sanitize_id
from typing import ClassVar, Dict, Optional, Union, cast, overload
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._application_fee import ApplicationFee
    from stripe._balance_transaction import BalanceTransaction
    from stripe._bank_account import BankAccount
    from stripe._card import Card
    from stripe.params._payout_cancel_params import PayoutCancelParams
    from stripe.params._payout_create_params import PayoutCreateParams
    from stripe.params._payout_list_params import PayoutListParams
    from stripe.params._payout_modify_params import PayoutModifyParams
    from stripe.params._payout_retrieve_params import PayoutRetrieveParams
    from stripe.params._payout_reverse_params import PayoutReverseParams


class Payout(
    CreateableAPIResource["Payout"],
    ListableAPIResource["Payout"],
    UpdateableAPIResource["Payout"],
):
    """
    A `Payout` object is created when you receive funds from Stripe, or when you
    initiate a payout to either a bank account or debit card of a [connected
    Stripe account](https://docs.stripe.com/docs/connect/bank-debit-card-payouts). You can retrieve individual payouts,
    and list all payouts. Payouts are made on [varying
    schedules](https://docs.stripe.com/docs/connect/manage-payout-schedule), depending on your country and
    industry.

    Related guide: [Receiving payouts](https://stripe.com/docs/payouts)
    """

    OBJECT_NAME: ClassVar[Literal["payout"]] = "payout"

    class TraceId(StripeObject):
        status: str
        """
        Possible values are `pending`, `supported`, and `unsupported`. When `payout.status` is `pending` or `in_transit`, this will be `pending`. When the payout transitions to `paid`, `failed`, or `canceled`, this status will become `supported` or `unsupported` shortly after in most cases. In some cases, this may appear as `pending` for up to 10 days after `arrival_date` until transitioning to `supported` or `unsupported`.
        """
        value: Optional[str]
        """
        The trace ID value if `trace_id.status` is `supported`, otherwise `nil`.
        """

    amount: int
    """
    The amount (in cents (or local equivalent)) that transfers to your bank account or debit card.
    """
    application_fee: Optional[ExpandableField["ApplicationFee"]]
    """
    The application fee (if any) for the payout. [See the Connect documentation](https://stripe.com/docs/connect/instant-payouts#monetization-and-fees) for details.
    """
    application_fee_amount: Optional[int]
    """
    The amount of the application fee (if any) requested for the payout. [See the Connect documentation](https://stripe.com/docs/connect/instant-payouts#monetization-and-fees) for details.
    """
    arrival_date: int
    """
    Date that you can expect the payout to arrive in the bank. This factors in delays to account for weekends or bank holidays.
    """
    automatic: bool
    """
    Returns `true` if the payout is created by an [automated payout schedule](https://stripe.com/docs/payouts#payout-schedule) and `false` if it's [requested manually](https://stripe.com/docs/payouts#manual-payouts).
    """
    balance_transaction: Optional[ExpandableField["BalanceTransaction"]]
    """
    ID of the balance transaction that describes the impact of this payout on your account balance.
    """
    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    currency: str
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    description: Optional[str]
    """
    An arbitrary string attached to the object. Often useful for displaying to users.
    """
    destination: Optional[ExpandableField[Union["BankAccount", "Card"]]]
    """
    ID of the bank account or card the payout is sent to.
    """
    failure_balance_transaction: Optional[
        ExpandableField["BalanceTransaction"]
    ]
    """
    If the payout fails or cancels, this is the ID of the balance transaction that reverses the initial balance transaction and returns the funds from the failed payout back in your balance.
    """
    failure_code: Optional[str]
    """
    Error code that provides a reason for a payout failure, if available. View our [list of failure codes](https://stripe.com/docs/api#payout_failures).
    """
    failure_message: Optional[str]
    """
    Message that provides the reason for a payout failure, if available.
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    metadata: Optional[Dict[str, str]]
    """
    Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    method: str
    """
    The method used to send this payout, which can be `standard` or `instant`. `instant` is supported for payouts to debit cards and bank accounts in certain countries. Learn more about [bank support for Instant Payouts](https://stripe.com/docs/payouts/instant-payouts-banks).
    """
    object: Literal["payout"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    original_payout: Optional[ExpandableField["Payout"]]
    """
    If the payout reverses another, this is the ID of the original payout.
    """
    payout_method: Optional[str]
    """
    ID of the v2 FinancialAccount the funds are sent to.
    """
    reconciliation_status: Literal[
        "completed", "in_progress", "not_applicable"
    ]
    """
    If `completed`, you can use the [Balance Transactions API](https://stripe.com/docs/api/balance_transactions/list#balance_transaction_list-payout) to list all balance transactions that are paid out in this payout.
    """
    reversed_by: Optional[ExpandableField["Payout"]]
    """
    If the payout reverses, this is the ID of the payout that reverses this payout.
    """
    source_type: str
    """
    The source balance this payout came from, which can be one of the following: `card`, `fpx`, or `bank_account`.
    """
    statement_descriptor: Optional[str]
    """
    Extra information about a payout that displays on the user's bank statement.
    """
    status: str
    """
    Current status of the payout: `paid`, `pending`, `in_transit`, `canceled` or `failed`. A payout is `pending` until it's submitted to the bank, when it becomes `in_transit`. The status changes to `paid` if the transaction succeeds, or to `failed` or `canceled` (within 5 business days). Some payouts that fail might initially show as `paid`, then change to `failed`.
    """
    trace_id: Optional[TraceId]
    """
    A value that generates from the beneficiary's bank that allows users to track payouts with their bank. Banks might call this a "reference number" or something similar.
    """
    type: Literal["bank_account", "card"]
    """
    Can be `bank_account` or `card`.
    """

    @classmethod
    def _cls_cancel(
        cls, payout: str, **params: Unpack["PayoutCancelParams"]
    ) -> "Payout":
        """
        You can cancel a previously created payout if its status is pending. Stripe refunds the funds to your available balance. You can't cancel automatic Stripe payouts.
        """
        return cast(
            "Payout",
            cls._static_request(
                "post",
                "/v1/payouts/{payout}/cancel".format(
                    payout=sanitize_id(payout)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    def cancel(
        payout: str, **params: Unpack["PayoutCancelParams"]
    ) -> "Payout":
        """
        You can cancel a previously created payout if its status is pending. Stripe refunds the funds to your available balance. You can't cancel automatic Stripe payouts.
        """
        ...

    @overload
    def cancel(self, **params: Unpack["PayoutCancelParams"]) -> "Payout":
        """
        You can cancel a previously created payout if its status is pending. Stripe refunds the funds to your available balance. You can't cancel automatic Stripe payouts.
        """
        ...

    @class_method_variant("_cls_cancel")
    def cancel(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["PayoutCancelParams"]
    ) -> "Payout":
        """
        You can cancel a previously created payout if its status is pending. Stripe refunds the funds to your available balance. You can't cancel automatic Stripe payouts.
        """
        return cast(
            "Payout",
            self._request(
                "post",
                "/v1/payouts/{payout}/cancel".format(
                    payout=sanitize_id(self.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    async def _cls_cancel_async(
        cls, payout: str, **params: Unpack["PayoutCancelParams"]
    ) -> "Payout":
        """
        You can cancel a previously created payout if its status is pending. Stripe refunds the funds to your available balance. You can't cancel automatic Stripe payouts.
        """
        return cast(
            "Payout",
            await cls._static_request_async(
                "post",
                "/v1/payouts/{payout}/cancel".format(
                    payout=sanitize_id(payout)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    async def cancel_async(
        payout: str, **params: Unpack["PayoutCancelParams"]
    ) -> "Payout":
        """
        You can cancel a previously created payout if its status is pending. Stripe refunds the funds to your available balance. You can't cancel automatic Stripe payouts.
        """
        ...

    @overload
    async def cancel_async(
        self, **params: Unpack["PayoutCancelParams"]
    ) -> "Payout":
        """
        You can cancel a previously created payout if its status is pending. Stripe refunds the funds to your available balance. You can't cancel automatic Stripe payouts.
        """
        ...

    @class_method_variant("_cls_cancel_async")
    async def cancel_async(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["PayoutCancelParams"]
    ) -> "Payout":
        """
        You can cancel a previously created payout if its status is pending. Stripe refunds the funds to your available balance. You can't cancel automatic Stripe payouts.
        """
        return cast(
            "Payout",
            await self._request_async(
                "post",
                "/v1/payouts/{payout}/cancel".format(
                    payout=sanitize_id(self.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    def create(cls, **params: Unpack["PayoutCreateParams"]) -> "Payout":
        """
        To send funds to your own bank account, create a new payout object. Your [Stripe balance](https://docs.stripe.com/api#balance) must cover the payout amount. If it doesn't, you receive an “Insufficient Funds” error.

        If your API key is in test mode, money won't actually be sent, though every other action occurs as if you're in live mode.

        If you create a manual payout on a Stripe account that uses multiple payment source types, you need to specify the source type balance that the payout draws from. The [balance object](https://docs.stripe.com/api#balance_object) details available and pending amounts by source type.
        """
        return cast(
            "Payout",
            cls._static_request(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    async def create_async(
        cls, **params: Unpack["PayoutCreateParams"]
    ) -> "Payout":
        """
        To send funds to your own bank account, create a new payout object. Your [Stripe balance](https://docs.stripe.com/api#balance) must cover the payout amount. If it doesn't, you receive an “Insufficient Funds” error.

        If your API key is in test mode, money won't actually be sent, though every other action occurs as if you're in live mode.

        If you create a manual payout on a Stripe account that uses multiple payment source types, you need to specify the source type balance that the payout draws from. The [balance object](https://docs.stripe.com/api#balance_object) details available and pending amounts by source type.
        """
        return cast(
            "Payout",
            await cls._static_request_async(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    def list(
        cls, **params: Unpack["PayoutListParams"]
    ) -> ListObject["Payout"]:
        """
        Returns a list of existing payouts sent to third-party bank accounts or payouts that Stripe sent to you. The payouts return in sorted order, with the most recently created payouts appearing first.
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    async def list_async(
        cls, **params: Unpack["PayoutListParams"]
    ) -> ListObject["Payout"]:
        """
        Returns a list of existing payouts sent to third-party bank accounts or payouts that Stripe sent to you. The payouts return in sorted order, with the most recently created payouts appearing first.
        """
        result = await cls._static_request_async(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def modify(
        cls, id: str, **params: Unpack["PayoutModifyParams"]
    ) -> "Payout":
        """
        Updates the specified payout by setting the values of the parameters you pass. We don't change parameters that you don't provide. This request only accepts the metadata as arguments.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "Payout",
            cls._static_request(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    async def modify_async(
        cls, id: str, **params: Unpack["PayoutModifyParams"]
    ) -> "Payout":
        """
        Updates the specified payout by setting the values of the parameters you pass. We don't change parameters that you don't provide. This request only accepts the metadata as arguments.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "Payout",
            await cls._static_request_async(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["PayoutRetrieveParams"]
    ) -> "Payout":
        """
        Retrieves the details of an existing payout. Supply the unique payout ID from either a payout creation request or the payout list. Stripe returns the corresponding payout information.
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["PayoutRetrieveParams"]
    ) -> "Payout":
        """
        Retrieves the details of an existing payout. Supply the unique payout ID from either a payout creation request or the payout list. Stripe returns the corresponding payout information.
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    @classmethod
    def _cls_reverse(
        cls, payout: str, **params: Unpack["PayoutReverseParams"]
    ) -> "Payout":
        """
        Reverses a payout by debiting the destination bank account. At this time, you can only reverse payouts for connected accounts to US and Canadian bank accounts. If the payout is manual and in the pending status, use /v1/payouts/:id/cancel instead.

        By requesting a reversal through /v1/payouts/:id/reverse, you confirm that the authorized signatory of the selected bank account authorizes the debit on the bank account and that no other authorization is required.
        """
        return cast(
            "Payout",
            cls._static_request(
                "post",
                "/v1/payouts/{payout}/reverse".format(
                    payout=sanitize_id(payout)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    def reverse(
        payout: str, **params: Unpack["PayoutReverseParams"]
    ) -> "Payout":
        """
        Reverses a payout by debiting the destination bank account. At this time, you can only reverse payouts for connected accounts to US and Canadian bank accounts. If the payout is manual and in the pending status, use /v1/payouts/:id/cancel instead.

        By requesting a reversal through /v1/payouts/:id/reverse, you confirm that the authorized signatory of the selected bank account authorizes the debit on the bank account and that no other authorization is required.
        """
        ...

    @overload
    def reverse(self, **params: Unpack["PayoutReverseParams"]) -> "Payout":
        """
        Reverses a payout by debiting the destination bank account. At this time, you can only reverse payouts for connected accounts to US and Canadian bank accounts. If the payout is manual and in the pending status, use /v1/payouts/:id/cancel instead.

        By requesting a reversal through /v1/payouts/:id/reverse, you confirm that the authorized signatory of the selected bank account authorizes the debit on the bank account and that no other authorization is required.
        """
        ...

    @class_method_variant("_cls_reverse")
    def reverse(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["PayoutReverseParams"]
    ) -> "Payout":
        """
        Reverses a payout by debiting the destination bank account. At this time, you can only reverse payouts for connected accounts to US and Canadian bank accounts. If the payout is manual and in the pending status, use /v1/payouts/:id/cancel instead.

        By requesting a reversal through /v1/payouts/:id/reverse, you confirm that the authorized signatory of the selected bank account authorizes the debit on the bank account and that no other authorization is required.
        """
        return cast(
            "Payout",
            self._request(
                "post",
                "/v1/payouts/{payout}/reverse".format(
                    payout=sanitize_id(self.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    async def _cls_reverse_async(
        cls, payout: str, **params: Unpack["PayoutReverseParams"]
    ) -> "Payout":
        """
        Reverses a payout by debiting the destination bank account. At this time, you can only reverse payouts for connected accounts to US and Canadian bank accounts. If the payout is manual and in the pending status, use /v1/payouts/:id/cancel instead.

        By requesting a reversal through /v1/payouts/:id/reverse, you confirm that the authorized signatory of the selected bank account authorizes the debit on the bank account and that no other authorization is required.
        """
        return cast(
            "Payout",
            await cls._static_request_async(
                "post",
                "/v1/payouts/{payout}/reverse".format(
                    payout=sanitize_id(payout)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    async def reverse_async(
        payout: str, **params: Unpack["PayoutReverseParams"]
    ) -> "Payout":
        """
        Reverses a payout by debiting the destination bank account. At this time, you can only reverse payouts for connected accounts to US and Canadian bank accounts. If the payout is manual and in the pending status, use /v1/payouts/:id/cancel instead.

        By requesting a reversal through /v1/payouts/:id/reverse, you confirm that the authorized signatory of the selected bank account authorizes the debit on the bank account and that no other authorization is required.
        """
        ...

    @overload
    async def reverse_async(
        self, **params: Unpack["PayoutReverseParams"]
    ) -> "Payout":
        """
        Reverses a payout by debiting the destination bank account. At this time, you can only reverse payouts for connected accounts to US and Canadian bank accounts. If the payout is manual and in the pending status, use /v1/payouts/:id/cancel instead.

        By requesting a reversal through /v1/payouts/:id/reverse, you confirm that the authorized signatory of the selected bank account authorizes the debit on the bank account and that no other authorization is required.
        """
        ...

    @class_method_variant("_cls_reverse_async")
    async def reverse_async(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["PayoutReverseParams"]
    ) -> "Payout":
        """
        Reverses a payout by debiting the destination bank account. At this time, you can only reverse payouts for connected accounts to US and Canadian bank accounts. If the payout is manual and in the pending status, use /v1/payouts/:id/cancel instead.

        By requesting a reversal through /v1/payouts/:id/reverse, you confirm that the authorized signatory of the selected bank account authorizes the debit on the bank account and that no other authorization is required.
        """
        return cast(
            "Payout",
            await self._request_async(
                "post",
                "/v1/payouts/{payout}/reverse".format(
                    payout=sanitize_id(self.get("id"))
                ),
                params=params,
            ),
        )

    _inner_class_types = {"trace_id": TraceId}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._list_object import ListObject
from stripe._stripe_object import StripeObject
from typing import ClassVar
from typing_extensions import Literal, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.entitlements._active_entitlement import ActiveEntitlement


class ActiveEntitlementSummary(StripeObject):
    """
    A summary of a customer's active entitlements.
    """

    OBJECT_NAME: ClassVar[
        Literal["entitlements.active_entitlement_summary"]
    ] = "entitlements.active_entitlement_summary"
    customer: str
    """
    The customer that is entitled to this feature.
    """
    entitlements: ListObject["ActiveEntitlement"]
    """
    The list of entitlements this customer has.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    object: Literal["entitlements.active_entitlement_summary"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe.financial_connections._account import Account
    from stripe.financial_connections._account_owner_service import (
        AccountOwnerService,
    )
    from stripe.params.financial_connections._account_disconnect_params import (
        AccountDisconnectParams,
    )
    from stripe.params.financial_connections._account_list_params import (
        AccountListParams,
    )
    from stripe.params.financial_connections._account_refresh_params import (
        AccountRefreshParams,
    )
    from stripe.params.financial_connections._account_retrieve_params import (
        AccountRetrieveParams,
    )
    from stripe.params.financial_connections._account_subscribe_params import (
        AccountSubscribeParams,
    )
    from stripe.params.financial_connections._account_unsubscribe_params import (
        AccountUnsubscribeParams,
    )

_subservices = {
    "owners": [
        "stripe.financial_connections._account_owner_service",
        "AccountOwnerService",
    ],
}


class AccountService(StripeService):
    owners: "AccountOwnerService"

    def __init__(self, requestor):
        super().__init__(requestor)

    def __getattr__(self, name):
        try:
            import_from, service = _subservices[name]
            service_class = getattr(
                import_module(import_from),
                service,
            )
            setattr(
                self,
                name,
                service_class(self._requestor),
            )
            return getattr(self, name)
        except KeyError:
            raise AttributeError()

    def list(
        self,
        params: Optional["AccountListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Account]":
        """
        Returns a list of Financial Connections Account objects.
        """
        return cast(
            "ListObject[Account]",
            self._request(
                "get",
                "/v1/financial_connections/accounts",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["AccountListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Account]":
        """
        Returns a list of Financial Connections Account objects.
        """
        return cast(
            "ListObject[Account]",
            await self._request_async(
                "get",
                "/v1/financial_connections/accounts",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        account: str,
        params: Optional["AccountRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Account":
        """
        Retrieves the details of an Financial Connections Account.
        """
        return cast(
            "Account",
            self._request(
                "get",
                "/v1/financial_connections/accounts/{account}".format(
                    account=sanitize_id(account),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        account: str,
        params: Optional["AccountRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Account":
        """
        Retrieves the details of an Financial Connections Account.
        """
        return cast(
            "Account",
            await self._request_async(
                "get",
                "/v1/financial_connections/accounts/{account}".format(
                    account=sanitize_id(account),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def disconnect(
        self,
        account: str,
        params: Optional["AccountDisconnectParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Account":
        """
        Disables your access to a Financial Connections Account. You will no longer be able to access data associated with the account (e.g. balances, transactions).
        """
        return cast(
            "Account",
            self._request(
                "post",
                "/v1/financial_connections/accounts/{account}/disconnect".format(
                    account=sanitize_id(account),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def disconnect_async(
        self,
        account: str,
        params: Optional["AccountDisconnectParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Account":
        """
        Disables your access to a Financial Connections Account. You will no longer be able to access data associated with the account (e.g. balances, transactions).
        """
        return cast(
            "Account",
            await self._request_async(
                "post",
                "/v1/financial_connections/accounts/{account}/disconnect".format(
                    account=sanitize_id(account),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def refresh(
        self,
        account: str,
        params: "AccountRefreshParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Account":
        """
        Refreshes the data associated with a Financial Connections Account.
        """
        return cast(
            "Account",
            self._request(
                "post",
                "/v1/financial_connections/accounts/{account}/refresh".format(
                    account=sanitize_id(account),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def refresh_async(
        self,
        account: str,
        params: "AccountRefreshParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Account":
        """
        Refreshes the data associated with a Financial Connections Account.
        """
        return cast(
            "Account",
            await self._request_async(
                "post",
                "/v1/financial_connections/accounts/{account}/refresh".format(
                    account=sanitize_id(account),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def subscribe(
        self,
        account: str,
        params: "AccountSubscribeParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Account":
        """
        Subscribes to periodic refreshes of data associated with a Financial Connections Account. When the account status is active, data is typically refreshed once a day.
        """
        return cast(
            "Account",
            self._request(
                "post",
                "/v1/financial_connections/accounts/{account}/subscribe".format(
                    account=sanitize_id(account),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def subscribe_async(
        self,
        account: str,
        params: "AccountSubscribeParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Account":
        """
        Subscribes to periodic refreshes of data associated with a Financial Connections Account. When the account status is active, data is typically refreshed once a day.
        """
        return cast(
            "Account",
            await self._request_async(
                "post",
                "/v1/financial_connections/accounts/{account}/subscribe".format(
                    account=sanitize_id(account),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def unsubscribe(
        self,
        account: str,
        params: "AccountUnsubscribeParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Account":
        """
        Unsubscribes from periodic refreshes of data associated with a Financial Connections Account.
        """
        return cast(
            "Account",
            self._request(
                "post",
                "/v1/financial_connections/accounts/{account}/unsubscribe".format(
                    account=sanitize_id(account),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def unsubscribe_async(
        self,
        account: str,
        params: "AccountUnsubscribeParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Account":
        """
        Unsubscribes from periodic refreshes of data associated with a Financial Connections Account.
        """
        return cast(
            "Account",
            await self._request_async(
                "post",
                "/v1/financial_connections/accounts/{account}/unsubscribe".format(
                    account=sanitize_id(account),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
from importlib import import_module
from typing import Dict, Tuple
from typing_extensions import TYPE_CHECKING, Type

from stripe._stripe_object import StripeObject

if TYPE_CHECKING:
    from stripe._api_mode import ApiMode

OBJECT_CLASSES: Dict[str, Tuple[str, str]] = {
    # data structures
    "list": ("stripe._list_object", "ListObject"),
    "search_result": ("stripe._search_result_object", "SearchResultObject"),
    "file": ("stripe._file", "File"),
    # there's also an alt name for compatibility
    "file_upload": ("stripe._file", "File"),
    # Object classes: The beginning of the section generated from our OpenAPI spec
    "account": ("stripe._account", "Account"),
    "account_link": ("stripe._account_link", "AccountLink"),
    "account_session": ("stripe._account_session", "AccountSession"),
    "apple_pay_domain": ("stripe._apple_pay_domain", "ApplePayDomain"),
    "application": ("stripe._application", "Application"),
    "application_fee": ("stripe._application_fee", "ApplicationFee"),
    "fee_refund": ("stripe._application_fee_refund", "ApplicationFeeRefund"),
    "apps.secret": ("stripe.apps._secret", "Secret"),
    "balance": ("stripe._balance", "Balance"),
    "balance_settings": ("stripe._balance_settings", "BalanceSettings"),
    "balance_transaction": (
        "stripe._balance_transaction",
        "BalanceTransaction",
    ),
    "bank_account": ("stripe._bank_account", "BankAccount"),
    "billing_portal.configuration": (
        "stripe.billing_portal._configuration",
        "Configuration",
    ),
    "billing_portal.session": ("stripe.billing_portal._session", "Session"),
    "billing.alert": ("stripe.billing._alert", "Alert"),
    "billing.alert_triggered": (
        "stripe.billing._alert_triggered",
        "AlertTriggered",
    ),
    "billing.credit_balance_summary": (
        "stripe.billing._credit_balance_summary",
        "CreditBalanceSummary",
    ),
    "billing.credit_balance_transaction": (
        "stripe.billing._credit_balance_transaction",
        "CreditBalanceTransaction",
    ),
    "billing.credit_grant": ("stripe.billing._credit_grant", "CreditGrant"),
    "billing.meter": ("stripe.billing._meter", "Meter"),
    "billing.meter_event": ("stripe.billing._meter_event", "MeterEvent"),
    "billing.meter_event_adjustment": (
        "stripe.billing._meter_event_adjustment",
        "MeterEventAdjustment",
    ),
    "billing.meter_event_summary": (
        "stripe.billing._meter_event_summary",
        "MeterEventSummary",
    ),
    "capability": ("stripe._capability", "Capability"),
    "card": ("stripe._card", "Card"),
    "cash_balance": ("stripe._cash_balance", "CashBalance"),
    "charge": ("stripe._charge", "Charge"),
    "checkout.session": ("stripe.checkout._session", "Session"),
    "climate.order": ("stripe.climate._order", "Order"),
    "climate.product": ("stripe.climate._product", "Product"),
    "climate.supplier": ("stripe.climate._supplier", "Supplier"),
    "confirmation_token": ("stripe._confirmation_token", "ConfirmationToken"),
    "connect_collection_transfer": (
        "stripe._connect_collection_transfer",
        "ConnectCollectionTransfer",
    ),
    "country_spec": ("stripe._country_spec", "CountrySpec"),
    "coupon": ("stripe._coupon", "Coupon"),
    "credit_note": ("stripe._credit_note", "CreditNote"),
    "credit_note_line_item": (
        "stripe._credit_note_line_item",
        "CreditNoteLineItem",
    ),
    "customer": ("stripe._customer", "Customer"),
    "customer_balance_transaction": (
        "stripe._customer_balance_transaction",
        "CustomerBalanceTransaction",
    ),
    "customer_cash_balance_transaction": (
        "stripe._customer_cash_balance_transaction",
        "CustomerCashBalanceTransaction",
    ),
    "customer_session": ("stripe._customer_session", "CustomerSession"),
    "discount": ("stripe._discount", "Discount"),
    "dispute": ("stripe._dispute", "Dispute"),
    "entitlements.active_entitlement": (
        "stripe.entitlements._active_entitlement",
        "ActiveEntitlement",
    ),
    "entitlements.active_entitlement_summary": (
        "stripe.entitlements._active_entitlement_summary",
        "ActiveEntitlementSummary",
    ),
    "entitlements.feature": ("stripe.entitlements._feature", "Feature"),
    "ephemeral_key": ("stripe._ephemeral_key", "EphemeralKey"),
    "event": ("stripe._event", "Event"),
    "exchange_rate": ("stripe._exchange_rate", "ExchangeRate"),
    "file": ("stripe._file", "File"),
    "file_link": ("stripe._file_link", "FileLink"),
    "financial_connections.account": (
        "stripe.financial_connections._account",
        "Account",
    ),
    "financial_connections.account_owner": (
        "stripe.financial_connections._account_owner",
        "AccountOwner",
    ),
    "financial_connections.account_ownership": (
        "stripe.financial_connections._account_ownership",
        "AccountOwnership",
    ),
    "financial_connections.session": (
        "stripe.financial_connections._session",
        "Session",
    ),
    "financial_connections.transaction": (
        "stripe.financial_connections._transaction",
        "Transaction",
    ),
    "forwarding.request": ("stripe.forwarding._request", "Request"),
    "funding_instructions": (
        "stripe._funding_instructions",
        "FundingInstructions",
    ),
    "identity.verification_report": (
        "stripe.identity._verification_report",
        "VerificationReport",
    ),
    "identity.verification_session": (
        "stripe.identity._verification_session",
        "VerificationSession",
    ),
    "invoice": ("stripe._invoice", "Invoice"),
    "invoiceitem": ("stripe._invoice_item", "InvoiceItem"),
    "line_item": ("stripe._invoice_line_item", "InvoiceLineItem"),
    "invoice_payment": ("stripe._invoice_payment", "InvoicePayment"),
    "invoice_rendering_template": (
        "stripe._invoice_rendering_template",
        "InvoiceRenderingTemplate",
    ),
    "issuing.authorization": (
        "stripe.issuing._authorization",
        "Authorization",
    ),
    "issuing.card": ("stripe.issuing._card", "Card"),
    "issuing.cardholder": ("stripe.issuing._cardholder", "Cardholder"),
    "issuing.dispute": ("stripe.issuing._dispute", "Dispute"),
    "issuing.personalization_design": (
        "stripe.issuing._personalization_design",
        "PersonalizationDesign",
    ),
    "issuing.physical_bundle": (
        "stripe.issuing._physical_bundle",
        "PhysicalBundle",
    ),
    "issuing.token": ("stripe.issuing._token", "Token"),
    "issuing.transaction": ("stripe.issuing._transaction", "Transaction"),
    "item": ("stripe._line_item", "LineItem"),
    "login_link": ("stripe._login_link", "LoginLink"),
    "mandate": ("stripe._mandate", "Mandate"),
    "payment_attempt_record": (
        "stripe._payment_attempt_record",
        "PaymentAttemptRecord",
    ),
    "payment_intent": ("stripe._payment_intent", "PaymentIntent"),
    "payment_intent_amount_details_line_item": (
        "stripe._payment_intent_amount_details_line_item",
        "PaymentIntentAmountDetailsLineItem",
    ),
    "payment_link": ("stripe._payment_link", "PaymentLink"),
    "payment_method": ("stripe._payment_method", "PaymentMethod"),
    "payment_method_configuration": (
        "stripe._payment_method_configuration",
        "PaymentMethodConfiguration",
    ),
    "payment_method_domain": (
        "stripe._payment_method_domain",
        "PaymentMethodDomain",
    ),
    "payment_record": ("stripe._payment_record", "PaymentRecord"),
    "payout": ("stripe._payout", "Payout"),
    "person": ("stripe._person", "Person"),
    "plan": ("stripe._plan", "Plan"),
    "price": ("stripe._price", "Price"),
    "product": ("stripe._product", "Product"),
    "product_feature": ("stripe._product_feature", "ProductFeature"),
    "promotion_code": ("stripe._promotion_code", "PromotionCode"),
    "quote": ("stripe._quote", "Quote"),
    "radar.early_fraud_warning": (
        "stripe.radar._early_fraud_warning",
        "EarlyFraudWarning",
    ),
    "radar.value_list": ("stripe.radar._value_list", "ValueList"),
    "radar.value_list_item": (
        "stripe.radar._value_list_item",
        "ValueListItem",
    ),
    "refund": ("stripe._refund", "Refund"),
    "reporting.report_run": ("stripe.reporting._report_run", "ReportRun"),
    "reporting.report_type": ("stripe.reporting._report_type", "ReportType"),
    "reserve_transaction": (
        "stripe._reserve_transaction",
        "ReserveTransaction",
    ),
    "transfer_reversal": ("stripe._reversal", "Reversal"),
    "review": ("stripe._review", "Review"),
    "setup_attempt": ("stripe._setup_attempt", "SetupAttempt"),
    "setup_intent": ("stripe._setup_intent", "SetupIntent"),
    "shipping_rate": ("stripe._shipping_rate", "ShippingRate"),
    "scheduled_query_run": (
        "stripe.sigma._scheduled_query_run",
        "ScheduledQueryRun",
    ),
    "source": ("stripe._source", "Source"),
    "source_mandate_notification": (
        "stripe._source_mandate_notification",
        "SourceMandateNotification",
    ),
    "source_transaction": ("stripe._source_transaction", "SourceTransaction"),
    "subscription": ("stripe._subscription", "Subscription"),
    "subscription_item": ("stripe._subscription_item", "SubscriptionItem"),
    "subscription_schedule": (
        "stripe._subscription_schedule",
        "SubscriptionSchedule",
    ),
    "tax.calculation": ("stripe.tax._calculation", "Calculation"),
    "tax.calculation_line_item": (
        "stripe.tax._calculation_line_item",
        "CalculationLineItem",
    ),
    "tax.registration": ("stripe.tax._registration", "Registration"),
    "tax.settings": ("stripe.tax._settings", "Settings"),
    "tax.transaction": ("stripe.tax._transaction", "Transaction"),
    "tax.transaction_line_item": (
        "stripe.tax._transaction_line_item",
        "TransactionLineItem",
    ),
    "tax_code": ("stripe._tax_code", "TaxCode"),
    "tax_deducted_at_source": (
        "stripe._tax_deducted_at_source",
        "TaxDeductedAtSource",
    ),
    "tax_id": ("stripe._tax_id", "TaxId"),
    "tax_rate": ("stripe._tax_rate", "TaxRate"),
    "terminal.configuration": (
        "stripe.terminal._configuration",
        "Configuration",
    ),
    "terminal.connection_token": (
        "stripe.terminal._connection_token",
        "ConnectionToken",
    ),
    "terminal.location": ("stripe.terminal._location", "Location"),
    "terminal.reader": ("stripe.terminal._reader", "Reader"),
    "test_helpers.test_clock": (
        "stripe.test_helpers._test_clock",
        "TestClock",
    ),
    "token": ("stripe._token", "Token"),
    "topup": ("stripe._topup", "Topup"),
    "transfer": ("stripe._transfer", "Transfer"),
    "treasury.credit_reversal": (
        "stripe.treasury._credit_reversal",
        "CreditReversal",
    ),
    "treasury.debit_reversal": (
        "stripe.treasury._debit_reversal",
        "DebitReversal",
    ),
    "treasury.financial_account": (
        "stripe.treasury._financial_account",
        "FinancialAccount",
    ),
    "treasury.financial_account_features": (
        "stripe.treasury._financial_account_features",
        "FinancialAccountFeatures",
    ),
    "treasury.inbound_transfer": (
        "stripe.treasury._inbound_transfer",
        "InboundTransfer",
    ),
    "treasury.outbound_payment": (
        "stripe.treasury._outbound_payment",
        "OutboundPayment",
    ),
    "treasury.outbound_transfer": (
        "stripe.treasury._outbound_transfer",
        "OutboundTransfer",
    ),
    "treasury.received_credit": (
        "stripe.treasury._received_credit",
        "ReceivedCredit",
    ),
    "treasury.received_debit": (
        "stripe.treasury._received_debit",
        "ReceivedDebit",
    ),
    "treasury.transaction": ("stripe.treasury._transaction", "Transaction"),
    "treasury.transaction_entry": (
        "stripe.treasury._transaction_entry",
        "TransactionEntry",
    ),
    "webhook_endpoint": ("stripe._webhook_endpoint", "WebhookEndpoint"),
    # Object classes: The end of the section generated from our OpenAPI spec
}

V2_OBJECT_CLASSES: Dict[str, Tuple[str, str]] = {
    # V2 Object classes: The beginning of the section generated from our OpenAPI spec
    "v2.billing.meter_event": ("stripe.v2.billing._meter_event", "MeterEvent"),
    "v2.billing.meter_event_adjustment": (
        "stripe.v2.billing._meter_event_adjustment",
        "MeterEventAdjustment",
    ),
    "v2.billing.meter_event_session": (
        "stripe.v2.billing._meter_event_session",
        "MeterEventSession",
    ),
    "v2.core.event": ("stripe.v2.core._event", "Event"),
    "v2.core.event_destination": (
        "stripe.v2.core._event_destination",
        "EventDestination",
    ),
    # V2 Object classes: The end of the section generated from our OpenAPI spec
}


def get_object_class(
    api_mode: "ApiMode", object_name: str
) -> Type[StripeObject]:
    mapping = OBJECT_CLASSES if api_mode == "V1" else V2_OBJECT_CLASSES

    if object_name not in mapping:
        return StripeObject

    import_path, class_name = mapping[object_name]
    return getattr(
        import_module(import_path),
        class_name,
    )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._list_object import ListObject
    from stripe._payment_method_configuration import PaymentMethodConfiguration
    from stripe._request_options import RequestOptions
    from stripe.params._payment_method_configuration_create_params import (
        PaymentMethodConfigurationCreateParams,
    )
    from stripe.params._payment_method_configuration_list_params import (
        PaymentMethodConfigurationListParams,
    )
    from stripe.params._payment_method_configuration_retrieve_params import (
        PaymentMethodConfigurationRetrieveParams,
    )
    from stripe.params._payment_method_configuration_update_params import (
        PaymentMethodConfigurationUpdateParams,
    )


class PaymentMethodConfigurationService(StripeService):
    def list(
        self,
        params: Optional["PaymentMethodConfigurationListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[PaymentMethodConfiguration]":
        """
        List payment method configurations
        """
        return cast(
            "ListObject[PaymentMethodConfiguration]",
            self._request(
                "get",
                "/v1/payment_method_configurations",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["PaymentMethodConfigurationListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[PaymentMethodConfiguration]":
        """
        List payment method configurations
        """
        return cast(
            "ListObject[PaymentMethodConfiguration]",
            await self._request_async(
                "get",
                "/v1/payment_method_configurations",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: Optional["PaymentMethodConfigurationCreateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentMethodConfiguration":
        """
        Creates a payment method configuration
        """
        return cast(
            "PaymentMethodConfiguration",
            self._request(
                "post",
                "/v1/payment_method_configurations",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: Optional["PaymentMethodConfigurationCreateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentMethodConfiguration":
        """
        Creates a payment method configuration
        """
        return cast(
            "PaymentMethodConfiguration",
            await self._request_async(
                "post",
                "/v1/payment_method_configurations",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        configuration: str,
        params: Optional["PaymentMethodConfigurationRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentMethodConfiguration":
        """
        Retrieve payment method configuration
        """
        return cast(
            "PaymentMethodConfiguration",
            self._request(
                "get",
                "/v1/payment_method_configurations/{configuration}".format(
                    configuration=sanitize_id(configuration),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        configuration: str,
        params: Optional["PaymentMethodConfigurationRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentMethodConfiguration":
        """
        Retrieve payment method configuration
        """
        return cast(
            "PaymentMethodConfiguration",
            await self._request_async(
                "get",
                "/v1/payment_method_configurations/{configuration}".format(
                    configuration=sanitize_id(configuration),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        configuration: str,
        params: Optional["PaymentMethodConfigurationUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentMethodConfiguration":
        """
        Update payment method configuration
        """
        return cast(
            "PaymentMethodConfiguration",
            self._request(
                "post",
                "/v1/payment_method_configurations/{configuration}".format(
                    configuration=sanitize_id(configuration),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        configuration: str,
        params: Optional["PaymentMethodConfigurationUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "PaymentMethodConfiguration":
        """
        Update payment method configuration
        """
        return cast(
            "PaymentMethodConfiguration",
            await self._request_async(
                "post",
                "/v1/payment_method_configurations/{configuration}".format(
                    configuration=sanitize_id(configuration),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._balance import Balance
    from stripe._request_options import RequestOptions
    from stripe.params._balance_retrieve_params import BalanceRetrieveParams


class BalanceService(StripeService):
    def retrieve(
        self,
        params: Optional["BalanceRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Balance":
        """
        Retrieves the current account balance, based on the authentication that was used to make the request.
         For a sample request, see [Accounting for negative balances](https://docs.stripe.com/docs/connect/account-balances#accounting-for-negative-balances).
        """
        return cast(
            "Balance",
            self._request(
                "get",
                "/v1/balance",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        params: Optional["BalanceRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Balance":
        """
        Retrieves the current account balance, based on the authentication that was used to make the request.
         For a sample request, see [Accounting for negative balances](https://docs.stripe.com/docs/connect/account-balances#accounting-for-negative-balances).
        """
        return cast(
            "Balance",
            await self._request_async(
                "get",
                "/v1/balance",
                base_address="api",
                params=params,
                options=options,
            ),
        )



import stripe


TEST_RESOURCE_ID = "ich_123"


class TestCardholder(object):
    def test_is_creatable(self, http_client_mock):
        resource = stripe.issuing.Cardholder.create(
            billing={
                "address": {
                    "city": "city",
                    "country": "US",
                    "line1": "line1",
                    "postal_code": "postal_code",
                }
            },
            name="Jenny Rosen",
            type="individual",
        )
        http_client_mock.assert_requested(
            "post",
            path="/v1/issuing/cardholders",
            post_data="billing[address][city]=city&billing[address][country]=US&billing[address][line1]=line1&billing[address][postal_code]=postal_code&name=Jenny+Rosen&type=individual",
        )
        assert isinstance(resource, stripe.issuing.Cardholder)

    def test_is_listable(self, http_client_mock):
        resources = stripe.issuing.Cardholder.list()
        http_client_mock.assert_requested(
            "get", path="/v1/issuing/cardholders"
        )
        assert isinstance(resources.data, list)
        assert isinstance(resources.data[0], stripe.issuing.Cardholder)

    def test_is_modifiable(self, http_client_mock):
        resource = stripe.issuing.Cardholder.modify(
            TEST_RESOURCE_ID, metadata={"key": "value"}
        )
        http_client_mock.assert_requested(
            "post",
            path="/v1/issuing/cardholders/%s" % TEST_RESOURCE_ID,
            post_data="metadata[key]=value",
        )
        assert isinstance(resource, stripe.issuing.Cardholder)

    def test_is_retrievable(self, http_client_mock):
        resource = stripe.issuing.Cardholder.retrieve(TEST_RESOURCE_ID)
        http_client_mock.assert_requested(
            "get", path="/v1/issuing/cardholders/%s" % TEST_RESOURCE_ID
        )
        assert isinstance(resource, stripe.issuing.Cardholder)

    def test_is_saveable(self, http_client_mock):
        resource = stripe.issuing.Cardholder.retrieve(TEST_RESOURCE_ID)
        resource.metadata["key"] = "value"
        cardholder = resource.save()
        http_client_mock.assert_requested(
            "post",
            path="/v1/issuing/cardholders/%s" % TEST_RESOURCE_ID,
            post_data="metadata[key]=value",
        )
        assert isinstance(resource, stripe.issuing.Cardholder)
        assert resource is cardholder



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import Dict, List
from typing_extensions import NotRequired, TypedDict


class OrderCreateParams(RequestOptions):
    amount: NotRequired[int]
    """
    Requested amount of carbon removal units. Either this or `metric_tons` must be specified.
    """
    beneficiary: NotRequired["OrderCreateParamsBeneficiary"]
    """
    Publicly sharable reference for the end beneficiary of carbon removal. Assumed to be the Stripe account if not set.
    """
    currency: NotRequired[str]
    """
    Request currency for the order as a three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a supported [settlement currency for your account](https://stripe.com/docs/currencies). If omitted, the account's default currency will be used.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    metadata: NotRequired[Dict[str, str]]
    """
    Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    metric_tons: NotRequired[str]
    """
    Requested number of tons for the order. Either this or `amount` must be specified.
    """
    product: str
    """
    Unique identifier of the Climate product.
    """


class OrderCreateParamsBeneficiary(TypedDict):
    public_name: str
    """
    Publicly displayable name for the end beneficiary of carbon removal.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe.climate._product import Product
    from stripe.params.climate._product_list_params import ProductListParams
    from stripe.params.climate._product_retrieve_params import (
        ProductRetrieveParams,
    )


class ProductService(StripeService):
    def list(
        self,
        params: Optional["ProductListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Product]":
        """
        Lists all available Climate product objects.
        """
        return cast(
            "ListObject[Product]",
            self._request(
                "get",
                "/v1/climate/products",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["ProductListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Product]":
        """
        Lists all available Climate product objects.
        """
        return cast(
            "ListObject[Product]",
            await self._request_async(
                "get",
                "/v1/climate/products",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        product: str,
        params: Optional["ProductRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Product":
        """
        Retrieves the details of a Climate product with the given ID.
        """
        return cast(
            "Product",
            self._request(
                "get",
                "/v1/climate/products/{product}".format(
                    product=sanitize_id(product),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        product: str,
        params: Optional["ProductRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Product":
        """
        Retrieves the details of a Climate product with the given ID.
        """
        return cast(
            "Product",
            await self._request_async(
                "get",
                "/v1/climate/products/{product}".format(
                    product=sanitize_id(product),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



import pytest
import json

from stripe._updateable_api_resource import UpdateableAPIResource


class TestUpdateableAPIResource(object):
    class MyUpdateable(UpdateableAPIResource):
        OBJECT_NAME = "myupdateable"

    @pytest.fixture
    def obj(self, http_client_mock):
        http_client_mock.stub_request(
            "post",
            path="/v1/myupdateables/myid",
            rbody=json.dumps({"id": "myid", "thats": "it"}),
            rheaders={"request-id": "req_id"},
        )

        return self.MyUpdateable.construct_from(
            {
                "id": "myid",
                "foo": "bar",
                "baz": "boz",
                "metadata": {"size": "l", "score": 4, "height": 10},
                "object": "obj",
            },
            "mykey",
        )

    def checkSave(self, obj):
        assert obj is obj.save()

        assert obj.thats == "it"
        # TODO: Should we force id to be retained?
        # assert obj.id == 'myid'
        with pytest.raises(AttributeError):
            obj.baz

    def test_idempotent_save(self, http_client_mock, obj):
        obj.baz = "updated"
        obj.save(idempotency_key="foo")

        http_client_mock.assert_requested(
            "post",
            path="/v1/myupdateables/myid",
            post_data="baz=updated",
            idempotency_key="foo",
        )

    def test_save(self, http_client_mock, obj):
        obj.baz = "updated"
        obj.other = "newval"
        obj.metadata.size = "m"
        obj.metadata.info = "a2"
        obj.metadata.height = None

        self.checkSave(obj)

        http_client_mock.assert_requested(
            "post",
            path="/v1/myupdateables/myid",
            post_data="baz=updated&metadata[info]=a2&metadata[size]=m&other=newval",
        )

        assert obj.last_response is not None
        assert obj.last_response.request_id == "req_id"

        # Saving again should not cause any request.
        http_client_mock.reset_mock()
        self.checkSave(obj)
        http_client_mock.assert_no_request()

        # Setting the same value should cause a request.
        http_client_mock.reset_mock()
        http_client_mock.stub_request(
            "post",
            path="/v1/myupdateables/myid",
            rbody=json.dumps({"id": "myid", "thats": "it"}),
        )

        obj.thats = "it"
        self.checkSave(obj)

        http_client_mock.assert_requested(
            "post", path="/v1/myupdateables/myid", post_data="thats=it"
        )

        # Changing the value should cause a request.
        http_client_mock.reset_mock()
        http_client_mock.stub_request(
            "post",
            path="/v1/myupdateables/myid",
            rbody=json.dumps({"id": "myid", "thats": "it"}),
        )

        obj.id = "myid"
        obj.thats = "updated"
        self.checkSave(obj)

        http_client_mock.assert_requested(
            "post", path="/v1/myupdateables/myid", post_data="thats=updated"
        )

    def test_add_key_to_nested_object(self, http_client_mock, obj):
        acct = self.MyUpdateable.construct_from(
            {
                "id": "myid",
                "legal_entity": {"size": "l", "score": 4, "height": 10},
            },
            "mykey",
        )

        acct.legal_entity["first_name"] = "bob"

        assert acct is acct.save()

        http_client_mock.assert_requested(
            "post",
            path="/v1/myupdateables/myid",
            post_data="legal_entity[first_name]=bob",
        )

    def test_save_nothing(self, http_client_mock, obj):
        acct = self.MyUpdateable.construct_from(
            {"id": "myid", "metadata": {"key": "value"}}, "mykey"
        )

        assert acct is acct.save()

        http_client_mock.assert_no_request()

    def test_replace_nested_object(self, http_client_mock, obj):
        acct = self.MyUpdateable.construct_from(
            {"id": "myid", "legal_entity": {"last_name": "smith"}}, "mykey"
        )

        acct.legal_entity = {"first_name": "bob"}

        assert acct is acct.save()

        http_client_mock.assert_requested(
            "post",
            path="/v1/myupdateables/myid",
            post_data="legal_entity[first_name]=bob&legal_entity[last_name]=",
        )

    def test_array_setting(self, http_client_mock, obj):
        acct = self.MyUpdateable.construct_from(
            {"id": "myid", "legal_entity": {}}, "mykey"
        )

        acct.legal_entity.additional_owners = [{"first_name": "Bob"}]

        assert acct is acct.save()

        http_client_mock.assert_requested(
            "post",
            path="/v1/myupdateables/myid",
            post_data="legal_entity[additional_owners][0][first_name]=Bob",
        )

    def test_array_none(self, http_client_mock, obj):
        acct = self.MyUpdateable.construct_from(
            {"id": "myid", "legal_entity": {"additional_owners": None}},
            "mykey",
        )

        acct.foo = "bar"

        assert acct is acct.save()

        http_client_mock.assert_requested(
            "post", path="/v1/myupdateables/myid", post_data="foo=bar"
        )

    def test_array_insertion(self, http_client_mock, obj):
        acct = self.MyUpdateable.construct_from(
            {"id": "myid", "legal_entity": {"additional_owners": []}}, "mykey"
        )

        acct.legal_entity.additional_owners.append({"first_name": "Bob"})

        assert acct is acct.save()

        http_client_mock.assert_requested(
            "post",
            path="/v1/myupdateables/myid",
            post_data="legal_entity[additional_owners][0][first_name]=Bob",
        )

    def test_array_update(self, http_client_mock, obj):
        acct = self.MyUpdateable.construct_from(
            {
                "id": "myid",
                "legal_entity": {
                    "additional_owners": [
                        {"first_name": "Bob"},
                        {"first_name": "Jane"},
                    ]
                },
            },
            "mykey",
        )

        acct.legal_entity.additional_owners[1].first_name = "Janet"

        assert acct is acct.save()

        http_client_mock.assert_requested(
            "post",
            path="/v1/myupdateables/myid",
            post_data="legal_entity[additional_owners][1][first_name]=Janet",
        )

    def test_array_noop(self, http_client_mock, obj):
        acct = self.MyUpdateable.construct_from(
            {
                "id": "myid",
                "legal_entity": {"additional_owners": [{"first_name": "Bob"}]},
                "currencies_supported": ["usd", "cad"],
            },
            "mykey",
        )

        assert acct is acct.save()

        http_client_mock.assert_requested(
            "post",
            path="/v1/myupdateables/myid",
            post_data="",
        )

    def test_hash_noop(self, http_client_mock, obj):
        acct = self.MyUpdateable.construct_from(
            {
                "id": "myid",
                "legal_entity": {"address": {"line1": "1 Two Three"}},
            },
            "mykey",
        )

        assert acct is acct.save()

        http_client_mock.assert_no_request()

    def test_save_replace_metadata_with_number(self, http_client_mock, obj):
        obj.baz = "updated"
        obj.other = "newval"
        obj.metadata = 3

        self.checkSave(obj)

        http_client_mock.assert_requested(
            "post",
            path="/v1/myupdateables/myid",
            post_data="baz=updated&metadata=3&other=newval",
        )

    def test_save_overwrite_metadata(self, http_client_mock, obj):
        obj.metadata = {}
        self.checkSave(obj)

        http_client_mock.assert_requested(
            "post",
            path="/v1/myupdateables/myid",
            post_data="metadata[size]=&metadata[score]=&metadata[height]=",
        )

    def test_save_replace_metadata(self, http_client_mock, obj):
        obj.baz = "updated"
        obj.other = "newval"
        obj.metadata = {"size": "m", "info": "a2", "score": 4}

        self.checkSave(obj)

        http_client_mock.assert_requested(
            "post",
            path="/v1/myupdateables/myid",
            post_data="baz=updated&metadata[info]=a2&metadata[size]=m&metadata[score]=4&other=newval",
        )

    def test_save_update_metadata(self, http_client_mock, obj):
        obj.baz = "updated"
        obj.other = "newval"
        obj.metadata.update({"size": "m", "info": "a2", "score": 4})

        self.checkSave(obj)

        http_client_mock.assert_requested(
            "post",
            path="/v1/myupdateables/myid",
            post_data="baz=updated&metadata[info]=a2&metadata[size]=m&metadata[score]=4&other=newval",
        )

    def test_retrieve_and_update_with_stripe_version(
        self, http_client_mock, obj
    ):
        http_client_mock.stub_request(
            "get",
            path="/v1/myupdateables/foo",
            rbody=json.dumps({"id": "foo", "bobble": "scrobble"}),
        )

        res = self.MyUpdateable.retrieve("foo", stripe_version="2017-08-15")

        http_client_mock.assert_requested(stripe_version="2017-08-15")

        http_client_mock.stub_request(
            "post",
            path="/v1/myupdateables/foo",
            rbody=json.dumps({"id": "foo", "bobble": "new_scrobble"}),
        )

        res.bobble = "new_scrobble"
        res.save()

        http_client_mock.assert_requested(stripe_version="2017-08-15")

    def test_modify_with_all_special_fields(self, http_client_mock, obj):
        http_client_mock.stub_request(
            "post",
            path="/v1/myupdateables/foo",
            rbody=json.dumps({"id": "foo", "bobble": "new_scrobble"}),
            rheaders={"Idempotency-Key": "IdempotencyKey"},
        )

        self.MyUpdateable.modify(
            "foo",
            stripe_version="2017-08-15",
            api_key="APIKEY",
            idempotency_key="IdempotencyKey",
            stripe_account="Acc",
            bobble="new_scrobble",
            headers={"extra_header": "val"},
        )

        http_client_mock.assert_requested(
            "post",
            path="/v1/myupdateables/foo",
            post_data="bobble=new_scrobble",
            stripe_version="2017-08-15",
            idempotency_key="IdempotencyKey",
            extra_headers={"extra_header": "val"},
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import Dict, List, Union
from typing_extensions import Literal, NotRequired, TypedDict


class ChargeModifyParams(RequestOptions):
    customer: NotRequired[str]
    """
    The ID of an existing customer that will be associated with this request. This field may only be updated if there is no existing associated customer with this charge.
    """
    description: NotRequired[str]
    """
    An arbitrary string which you can attach to a charge object. It is displayed when in the web interface alongside the charge. Note that if you use Stripe to send automatic email receipts to your customers, your receipt emails will include the `description` of the charge(s) that they are describing.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    fraud_details: NotRequired["ChargeModifyParamsFraudDetails"]
    """
    A set of key-value pairs you can attach to a charge giving information about its riskiness. If you believe a charge is fraudulent, include a `user_report` key with a value of `fraudulent`. If you believe a charge is safe, include a `user_report` key with a value of `safe`. Stripe will use the information you send to improve our fraud detection algorithms.
    """
    metadata: NotRequired["Literal['']|Dict[str, str]"]
    """
    Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    receipt_email: NotRequired[str]
    """
    This is the email address that the receipt for this charge will be sent to. If this field is updated, then a new email receipt will be sent to the updated address.
    """
    shipping: NotRequired["ChargeModifyParamsShipping"]
    """
    Shipping information for the charge. Helps prevent fraud on charges for physical goods.
    """
    transfer_group: NotRequired[str]
    """
    A string that identifies this transaction as part of a group. `transfer_group` may only be provided if it has not been set. See the [Connect documentation](https://stripe.com/docs/connect/separate-charges-and-transfers#transfer-options) for details.
    """


class ChargeModifyParamsFraudDetails(TypedDict):
    user_report: Union[Literal[""], Literal["fraudulent", "safe"]]
    """
    Either `safe` or `fraudulent`.
    """


class ChargeModifyParamsShipping(TypedDict):
    address: "ChargeModifyParamsShippingAddress"
    """
    Shipping address.
    """
    carrier: NotRequired[str]
    """
    The delivery service that shipped a physical product, such as Fedex, UPS, USPS, etc.
    """
    name: str
    """
    Recipient name.
    """
    phone: NotRequired[str]
    """
    Recipient phone (including extension).
    """
    tracking_number: NotRequired[str]
    """
    The tracking number for a physical product, obtained from the delivery service. If multiple tracking numbers were generated for this purchase, please separate them with commas.
    """


class ChargeModifyParamsShippingAddress(TypedDict):
    city: NotRequired[str]
    """
    City, district, suburb, town, or village.
    """
    country: NotRequired[str]
    """
    Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
    """
    line1: NotRequired[str]
    """
    Address line 1, such as the street, PO Box, or company name.
    """
    line2: NotRequired[str]
    """
    Address line 2, such as the apartment, suite, unit, or building.
    """
    postal_code: NotRequired[str]
    """
    ZIP or postal code.
    """
    state: NotRequired[str]
    """
    State, county, province, or region.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._list_object import ListObject
from stripe._listable_api_resource import ListableAPIResource
from stripe._stripe_object import StripeObject
from typing import ClassVar, Dict, List, Optional
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.climate._supplier import Supplier
    from stripe.params.climate._product_list_params import ProductListParams
    from stripe.params.climate._product_retrieve_params import (
        ProductRetrieveParams,
    )


class Product(ListableAPIResource["Product"]):
    """
    A Climate product represents a type of carbon removal unit available for reservation.
    You can retrieve it to see the current price and availability.
    """

    OBJECT_NAME: ClassVar[Literal["climate.product"]] = "climate.product"

    class CurrentPricesPerMetricTon(StripeObject):
        amount_fees: int
        """
        Fees for one metric ton of carbon removal in the currency's smallest unit.
        """
        amount_subtotal: int
        """
        Subtotal for one metric ton of carbon removal (excluding fees) in the currency's smallest unit.
        """
        amount_total: int
        """
        Total for one metric ton of carbon removal (including fees) in the currency's smallest unit.
        """

    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    current_prices_per_metric_ton: Dict[str, CurrentPricesPerMetricTon]
    """
    Current prices for a metric ton of carbon removal in a currency's smallest unit.
    """
    delivery_year: Optional[int]
    """
    The year in which the carbon removal is expected to be delivered.
    """
    id: str
    """
    Unique identifier for the object. For convenience, Climate product IDs are human-readable strings
    that start with `climsku_`. See [carbon removal inventory](https://stripe.com/docs/climate/orders/carbon-removal-inventory)
    for a list of available carbon removal products.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    metric_tons_available: str
    """
    The quantity of metric tons available for reservation.
    """
    name: str
    """
    The Climate product's name.
    """
    object: Literal["climate.product"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    suppliers: List["Supplier"]
    """
    The carbon removal suppliers that fulfill orders for this Climate product.
    """

    @classmethod
    def list(
        cls, **params: Unpack["ProductListParams"]
    ) -> ListObject["Product"]:
        """
        Lists all available Climate product objects.
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    async def list_async(
        cls, **params: Unpack["ProductListParams"]
    ) -> ListObject["Product"]:
        """
        Lists all available Climate product objects.
        """
        result = await cls._static_request_async(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["ProductRetrieveParams"]
    ) -> "Product":
        """
        Retrieves the details of a Climate product with the given ID.
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["ProductRetrieveParams"]
    ) -> "Product":
        """
        Retrieves the details of a Climate product with the given ID.
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    _inner_class_types = {
        "current_prices_per_metric_ton": CurrentPricesPerMetricTon,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.params.test_helpers.treasury._outbound_transfer_fail_params import (
        OutboundTransferFailParams,
    )
    from stripe.params.test_helpers.treasury._outbound_transfer_post_params import (
        OutboundTransferPostParams,
    )
    from stripe.params.test_helpers.treasury._outbound_transfer_return_outbound_transfer_params import (
        OutboundTransferReturnOutboundTransferParams,
    )
    from stripe.params.test_helpers.treasury._outbound_transfer_update_params import (
        OutboundTransferUpdateParams,
    )
    from stripe.treasury._outbound_transfer import OutboundTransfer


class OutboundTransferService(StripeService):
    def update(
        self,
        outbound_transfer: str,
        params: "OutboundTransferUpdateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "OutboundTransfer":
        """
        Updates a test mode created OutboundTransfer with tracking details. The OutboundTransfer must not be cancelable, and cannot be in the canceled or failed states.
        """
        return cast(
            "OutboundTransfer",
            self._request(
                "post",
                "/v1/test_helpers/treasury/outbound_transfers/{outbound_transfer}".format(
                    outbound_transfer=sanitize_id(outbound_transfer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        outbound_transfer: str,
        params: "OutboundTransferUpdateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "OutboundTransfer":
        """
        Updates a test mode created OutboundTransfer with tracking details. The OutboundTransfer must not be cancelable, and cannot be in the canceled or failed states.
        """
        return cast(
            "OutboundTransfer",
            await self._request_async(
                "post",
                "/v1/test_helpers/treasury/outbound_transfers/{outbound_transfer}".format(
                    outbound_transfer=sanitize_id(outbound_transfer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def fail(
        self,
        outbound_transfer: str,
        params: Optional["OutboundTransferFailParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "OutboundTransfer":
        """
        Transitions a test mode created OutboundTransfer to the failed status. The OutboundTransfer must already be in the processing state.
        """
        return cast(
            "OutboundTransfer",
            self._request(
                "post",
                "/v1/test_helpers/treasury/outbound_transfers/{outbound_transfer}/fail".format(
                    outbound_transfer=sanitize_id(outbound_transfer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def fail_async(
        self,
        outbound_transfer: str,
        params: Optional["OutboundTransferFailParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "OutboundTransfer":
        """
        Transitions a test mode created OutboundTransfer to the failed status. The OutboundTransfer must already be in the processing state.
        """
        return cast(
            "OutboundTransfer",
            await self._request_async(
                "post",
                "/v1/test_helpers/treasury/outbound_transfers/{outbound_transfer}/fail".format(
                    outbound_transfer=sanitize_id(outbound_transfer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def post(
        self,
        outbound_transfer: str,
        params: Optional["OutboundTransferPostParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "OutboundTransfer":
        """
        Transitions a test mode created OutboundTransfer to the posted status. The OutboundTransfer must already be in the processing state.
        """
        return cast(
            "OutboundTransfer",
            self._request(
                "post",
                "/v1/test_helpers/treasury/outbound_transfers/{outbound_transfer}/post".format(
                    outbound_transfer=sanitize_id(outbound_transfer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def post_async(
        self,
        outbound_transfer: str,
        params: Optional["OutboundTransferPostParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "OutboundTransfer":
        """
        Transitions a test mode created OutboundTransfer to the posted status. The OutboundTransfer must already be in the processing state.
        """
        return cast(
            "OutboundTransfer",
            await self._request_async(
                "post",
                "/v1/test_helpers/treasury/outbound_transfers/{outbound_transfer}/post".format(
                    outbound_transfer=sanitize_id(outbound_transfer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def return_outbound_transfer(
        self,
        outbound_transfer: str,
        params: Optional[
            "OutboundTransferReturnOutboundTransferParams"
        ] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "OutboundTransfer":
        """
        Transitions a test mode created OutboundTransfer to the returned status. The OutboundTransfer must already be in the processing state.
        """
        return cast(
            "OutboundTransfer",
            self._request(
                "post",
                "/v1/test_helpers/treasury/outbound_transfers/{outbound_transfer}/return".format(
                    outbound_transfer=sanitize_id(outbound_transfer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def return_outbound_transfer_async(
        self,
        outbound_transfer: str,
        params: Optional[
            "OutboundTransferReturnOutboundTransferParams"
        ] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "OutboundTransfer":
        """
        Transitions a test mode created OutboundTransfer to the returned status. The OutboundTransfer must already be in the processing state.
        """
        return cast(
            "OutboundTransfer",
            await self._request_async(
                "post",
                "/v1/test_helpers/treasury/outbound_transfers/{outbound_transfer}/return".format(
                    outbound_transfer=sanitize_id(outbound_transfer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import List
from typing_extensions import Literal, NotRequired, TypedDict


class OutboundPaymentReturnOutboundPaymentParams(RequestOptions):
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    returned_details: NotRequired[
        "OutboundPaymentReturnOutboundPaymentParamsReturnedDetails"
    ]
    """
    Optional hash to set the return code.
    """


class OutboundPaymentReturnOutboundPaymentParamsReturnedDetails(TypedDict):
    code: NotRequired[
        Literal[
            "account_closed",
            "account_frozen",
            "bank_account_restricted",
            "bank_ownership_changed",
            "declined",
            "incorrect_account_holder_name",
            "invalid_account_number",
            "invalid_currency",
            "no_account",
            "other",
        ]
    ]
    """
    The return code to be set on the OutboundPayment object.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._customer_cash_balance_transaction import (
        CustomerCashBalanceTransaction,
    )
    from stripe._request_options import RequestOptions
    from stripe.params.test_helpers._customer_fund_cash_balance_params import (
        CustomerFundCashBalanceParams,
    )


class CustomerService(StripeService):
    def fund_cash_balance(
        self,
        customer: str,
        params: "CustomerFundCashBalanceParams",
        options: Optional["RequestOptions"] = None,
    ) -> "CustomerCashBalanceTransaction":
        """
        Create an incoming testmode bank transfer
        """
        return cast(
            "CustomerCashBalanceTransaction",
            self._request(
                "post",
                "/v1/test_helpers/customers/{customer}/fund_cash_balance".format(
                    customer=sanitize_id(customer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def fund_cash_balance_async(
        self,
        customer: str,
        params: "CustomerFundCashBalanceParams",
        options: Optional["RequestOptions"] = None,
    ) -> "CustomerCashBalanceTransaction":
        """
        Create an incoming testmode bank transfer
        """
        return cast(
            "CustomerCashBalanceTransaction",
            await self._request_async(
                "post",
                "/v1/test_helpers/customers/{customer}/fund_cash_balance".format(
                    customer=sanitize_id(customer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )
